@@ -24,6 +24,6 @@
 
 #include	"brlmonxml.h"
 
-void brlmon_refresh	(void);
+void brlmon_refresh_display ();
 
 #endif
