@@ -94,7 +94,7 @@ CmdMapFunctionsList cmd_function[] =
 	{"toggle tracking mode",N_("toggle flat review/focus tracking mode")},	
 	{"flat review", 	N_("flat review")},
 	{"window hierarchy", 	N_("window hierarchy")},
-	{"window hierarchy flat", 	N_("window hierarchy flat")},
+	{"read whole window", 	N_("read whole window")},
 	{"detailed informations", N_("detailed informations")},
 	{"do default action", 	N_("do default action")},
 	{"window overview", 	N_("window overview")},
@@ -1450,7 +1450,7 @@ cmdui_main_window_response_event (GtkDialog *dialog,
 	}
         break;
     case GTK_RESPONSE_HELP:
-	    gn_load_help ("gnopernicus-command-mapping");
+	    gn_load_help ("gnopernicus-cm-prefs");
 	break;	
     case GTK_RESPONSE_CANCEL:
     default:
