@@ -19,11 +19,13 @@
  * Boston, MA 02111-1307, USA.
  */
 
+#include "config.h"
 #include "libsrconf.h"
 #include <gconf/gconf-client.h>
 #include <stdio.h>
 #include <string.h>
 #include "SRMessages.h"
+#include "srintl.h"
 
 
 /**
@@ -63,6 +65,7 @@ static SRConfStatusEnum srconf_status = SRCONF_IDLE;
  *
 **/
 GConfClient *gconf_client=NULL;
+GConfEngine *gconf_engine=NULL;
 
 /**
  * gconf_root_dir_path:
@@ -72,6 +75,7 @@ GConfClient *gconf_client=NULL;
 **/
 static gchar *gconf_root_dir_path = NULL;
 
+static gboolean use_config_settings = FALSE;
 /**
  * DEFAULT_ROOT_DIR_PATH:
  *
@@ -232,8 +236,7 @@ sr_config_structure_destructor (gpointer data)
  *
 **/
 static void 
-sr_config_changed_callback (GConfClient *client, 
-			    guint cnxn_id, 
+sr_config_changed_callback (guint cnxn_id, 
 			    GConfEntry *entry, 
 			    gpointer user_data)
 {
@@ -241,35 +244,36 @@ sr_config_changed_callback (GConfClient *client,
     SRConfigStructure *SRstruct = NULL;
 
     sru_return_if_fail (entry);
-    sru_return_if_fail (entry->value);
+    if (!entry->value)
+	return;
     
     SRstruct = (SRConfigStructure*) g_new0 (SRConfigStructure, 1);
     sru_assert (SRstruct);
 
-    sru_debug("srconf:Entry key:%s", gconf_entry_get_key (entry));
+    sru_debug("Entry key:%s", gconf_entry_get_key (entry));
     if (entry->value)
     {
-	sru_debug("srconf:Entry value type:%i",entry->value->type);
+	sru_debug("Entry value type:%i",entry->value->type);
 	switch(entry->value->type)
 	{	
     	    case GCONF_VALUE_INT:    
-		sru_debug("srconf:Entry value:%i",gconf_value_get_int(entry->value));
+		sru_debug("Entry value:%i",gconf_value_get_int(entry->value));
 		break;
     	    case GCONF_VALUE_FLOAT:  
-		sru_debug("srconf:Entry value:%f",gconf_value_get_float(entry->value));
+		sru_debug("Entry value:%f",gconf_value_get_float(entry->value));
 		break;
     	    case GCONF_VALUE_STRING: 
-		sru_debug ("srconf:Entry value:%s",gconf_value_get_string(entry->value));
+		sru_debug ("Entry value:%s",gconf_value_get_string(entry->value));
 		break;
     	    case GCONF_VALUE_BOOL:   
-		sru_debug ("srconf:Entry value:%i",gconf_value_get_bool(entry->value));
+		sru_debug ("Entry value:%i",gconf_value_get_bool(entry->value));
 		break;
     	    case GCONF_VALUE_LIST: 
-	    	sru_debug ("srconf:list");
+	    	sru_debug ("list");
 		break;
 	    case GCONF_VALUE_INVALID:
     	    default:
-	    	sru_debug ("srconf:unset");
+	    	sru_debug ("unset");
 		break;
 	}
     }
@@ -301,8 +305,8 @@ sr_config_changed_callback (GConfClient *client,
 		break;
     	    case GCONF_VALUE_FLOAT:
 		SRstruct->type = CFGT_FLOAT;
-		SRstruct->newvalue = (gpointer)g_new0 (gfloat, 1);
-		*((gfloat*)SRstruct->newvalue) = gconf_value_get_float (entry->value);
+		SRstruct->newvalue = (gpointer)g_new0 (gdouble, 1);
+		*((gdouble*)SRstruct->newvalue) = gconf_value_get_float (entry->value);
 		break;
     	    case GCONF_VALUE_STRING:
 		SRstruct->type = CFGT_STRING;
@@ -356,8 +360,26 @@ sr_config_changed_callback (GConfClient *client,
         srconf_fnc (evnt,0);
 
     sre_release_reference (evnt);
-    sru_debug ("srconf:SendEventSrCore");
+    sru_debug ("SendEventSrCore");
 }
+
+static void 
+sr_config_client_changed_callback (GConfClient *client, 
+			    guint cnxn_id, 
+			    GConfEntry *entry, 
+			    gpointer user_data)
+{
+    sr_config_changed_callback (cnxn_id, entry, user_data);
+}
+static void 
+sr_config_engine_changed_callback (GConfEngine *engine, 
+			    guint cnxn_id, 
+			    GConfEntry *entry, 
+			    gpointer user_data)
+{
+    sr_config_changed_callback (cnxn_id, entry, user_data);
+}
+
 /**
  * srconf_init:
  *
@@ -366,7 +388,9 @@ sr_config_changed_callback (GConfClient *client,
  *	gconf_root_dir_path - the root directory of GConf
 **/
 gboolean 
-srconf_init (SRConfCB srconfcb, const gchar *_gconf_root_dir_path)
+srconf_init (SRConfCB srconfcb, 
+	     const gchar *_gconf_root_dir_path,
+	     const gchar *config_source)
 {
     gint i;
     GError *error = NULL;    
@@ -374,52 +398,93 @@ srconf_init (SRConfCB srconfcb, const gchar *_gconf_root_dir_path)
     
     sru_return_val_if_fail (srconf_status == SRCONF_IDLE, FALSE);
     sru_return_val_if_fail (srconfcb != NULL, FALSE);
-
+    
     srconf_fnc = srconfcb;
 
     gconf_root_dir_path = g_strdup (_gconf_root_dir_path?_gconf_root_dir_path: DEFAULT_ROOT_DIR_PATH);
     
     sru_return_val_if_fail (gconf_root_dir_path != NULL, FALSE);
 
-    /*set a default gconf client		      */
-    gconf_client = gconf_client_get_default ();
-    
-    /*adding the directory to be watched		      */
-    gconf_client_add_dir (gconf_client, gconf_root_dir_path, GCONF_CLIENT_PRELOAD_ONELEVEL, &error);
-    
-    
-    if (error != NULL)
-    {
-	sru_warning("srconf:Failed to add dir.\n");
-	sru_warning("Recomanded to delete ~/.gconf/apps/gnopernicus and ~/.gconfd/lock directories");
-	g_error_free(error);
-	error = NULL;
-    }
-	
-    /*adding notification callbacks*/
-    for( i=0; i < NUM_OF_CONFIGURABLES; i++ )
-    {
-    	path = g_strdup_printf ("%s%s", gconf_root_dir_path, srconf_notify_directories[i].notify_path);
-    	srconf_notify_directories[i].cnx_id = gconf_client_notify_add (gconf_client, path, sr_config_changed_callback, (gpointer)(i), NULL, &error);
+    /*set a default gconf client */
+    if (!config_source)
+    {	
+	use_config_settings = FALSE;
+        gconf_client = gconf_client_get_default ();
 	
+	/*adding the directory to be watched*/
+	gconf_client_add_dir (gconf_client, gconf_root_dir_path, GCONF_CLIENT_PRELOAD_ONELEVEL, &error);
 	if (error != NULL)
 	{
-	    sru_warning ("srconf:Failed to add notify.\n");
-	    sru_warning ("Recomanded to delete ~/.gconf/apps/gnopernicus and ~/.gconfd/lock directories");
-	    g_error_free (error);
+	    sru_warning(_("Failed to add directory."));
+	    sru_warning(_(error->message));
+	    g_error_free(error);
 	    error = NULL;
 	}
+	
+	for( i=0; i < NUM_OF_CONFIGURABLES; i++ )
+	{
+    	    path = g_strdup_printf ("%s%s", gconf_root_dir_path, srconf_notify_directories[i].notify_path);
+    	    srconf_notify_directories[i].cnx_id = gconf_client_notify_add (gconf_client, path, sr_config_client_changed_callback, (gpointer)(i), NULL, &error);
+	
+	    if (error != NULL)
+	    {
+		sru_warning (_("Failed to add notify."));
+		sru_warning (_(error->message));
+		g_error_free (error);
+		error = NULL;
+	    }
 
-    	g_free (path);
-    	path = NULL;
+    	    g_free (path);
+    	    path = NULL;
+	}
+
+    }
+    else
+    {
+	path = g_strdup_printf ("xml:readwrite:%s", config_source);
+    	gconf_engine = gconf_engine_get_for_address (path, &error);
+	g_free (path);
+	
+	use_config_settings = TRUE;
+/*      conf = gconf_engine_get_local (config_source, &error);*/
+	
+	if (gconf_engine == NULL)
+	{
+    	    sru_assert (error != NULL);
+    	    sru_warning(_("Failed to access configuration source(s): %s\n"), error->message);
+    	    g_error_free (error);
+    	    error = NULL;
+    	    return FALSE;
+	}
+	
+	for( i=0; i < NUM_OF_CONFIGURABLES; i++ )
+	{
+    	    path = g_strdup_printf ("%s%s", gconf_root_dir_path, srconf_notify_directories[i].notify_path);
+    	    srconf_notify_directories[i].cnx_id = gconf_engine_notify_add (gconf_engine, path, sr_config_engine_changed_callback, (gpointer)(i),  &error);
+	
+	    if (error != NULL)
+	    {
+		sru_warning(_("Failed to add notify: %s"), error->message);
+		sru_warning(_(error->message));
+		g_error_free (error);
+		error = NULL;
+	    }	
+
+    	    g_free (path);
+    	    path = NULL;
+	}
+	
+	gconf_client = gconf_client_get_for_engine (gconf_engine);
+	
+	gconf_engine_unref (gconf_engine);
     }
         
+    
     srconf_status = SRCONF_RUNNING;
     sru_debug ("SRConf running.");
             
     return TRUE;
 }
-
 /**
  * srconf_terminate:
  *
@@ -433,18 +498,29 @@ srconf_terminate()
     GError *error = NULL;
     sru_return_if_fail (srconf_status == SRCONF_RUNNING);
     
-    /* removing notification callbacks */
-    for( i=0; i < NUM_OF_CONFIGURABLES; i++ )
-    	gconf_client_notify_remove (gconf_client, srconf_notify_directories[i].cnx_id);
+    if (!use_config_settings)
+    {
+	/* removing notification callbacks */
+	for( i=0; i < NUM_OF_CONFIGURABLES; i++ )
+    	    gconf_client_notify_remove (gconf_client, srconf_notify_directories[i].cnx_id);
     	
-    /* remove the watching directory */
-    gconf_client_remove_dir (gconf_client, gconf_root_dir_path, &error);
+	/* remove the watching directory */
+	gconf_client_remove_dir (gconf_client, gconf_root_dir_path, &error);
+    }
+    else
+    {
+	/* removing notification callbacks */
+	for( i=0; i < NUM_OF_CONFIGURABLES; i++ )
+    	    gconf_engine_notify_remove (gconf_engine, srconf_notify_directories[i].cnx_id);
+	    
+	gconf_engine_unref (gconf_engine);
+    }
     
     if (error != NULL)
     {
-	sru_warning("srconf:Failed to remove dir.\n");
-	sru_warning("Recomanded to delete ~/.gconf/apps/gnopernicus and ~/.gconfd/lock directories");
-	g_error_free(error);
+	sru_warning (_("Failed to remove directory."));
+	sru_warning (_(error->message));
+	g_error_free (error);
 	error = NULL;
     }
 
@@ -549,6 +625,8 @@ srconf_get_config_data_with_default(const gchar *key, SRConfigTypesEnum conftype
 		*((gpointer*)data) = NULL;
 	        used_default = FALSE;
 	    }
+	    
+	    gconf_value_free (value);
 	}
 	else
 	{
@@ -677,6 +755,8 @@ srconf_get_data_with_default(const gchar *key, SRConfigTypesEnum conftype, gpoin
 		*((gpointer*)data) = NULL;
 	        used_default = FALSE;
 	    }
+	    
+	    gconf_value_free (value);
 	}
 	else
 	{
@@ -782,8 +862,8 @@ srconf_set_config_data(const gchar *key, SRConfigTypesEnum conftype, const gpoin
     
     if (error != NULL)
     {
-        sru_warning ("srconf:Failed to set configdata");
-        sru_warning ("Recomanded to delete ~/.gconf/apps/gnopernicus and ~/.gconfd/lock directories");
+        sru_warning (_("Failed to set configdata."));
+        sru_warning (_(error->message));
         g_error_free (error);
         error = NULL;
         ret = FALSE;
@@ -856,8 +936,8 @@ srconf_set_data(const gchar *key, SRConfigTypesEnum conftype, const gpointer dat
     
     if (error != NULL)
     {
-        sru_warning ("srconf:Failed set data");
-        sru_warning ("Recomanded to delete ~/.gconf/apps/gnopernicus and ~/.gconfd/lock directories");
+        sru_warning (_("Failed set data."));
+        sru_warning (_(error->message));
         g_error_free(error);
         error = NULL;
         ret = FALSE;
@@ -878,7 +958,7 @@ srconf_unset_key (const gchar *key,const gchar *section)
     GError *error = NULL;
     GConfValue *val = NULL;
     gchar *path = NULL;
-    gboolean retval;
+    gboolean retval = TRUE;
     
     sru_return_val_if_fail (section != NULL, FALSE);
     sru_return_val_if_fail (key != NULL, FALSE);
@@ -895,7 +975,7 @@ srconf_unset_key (const gchar *key,const gchar *section)
         
 	if (error != NULL)
 	{
-	    sru_warning ("srconf:Failed unset key");
+	    sru_warning ("Failed unset key.");
 	    g_error_free(error);
 	    error = NULL;
 	}
@@ -905,3 +985,22 @@ srconf_unset_key (const gchar *key,const gchar *section)
     
     return retval;
 }
+
+gchar*
+srconf_presentationi_get_chunk (const gchar *device_role_event)
+{
+    gchar *active = NULL;
+    gchar *chunk  = NULL;
+    
+    sru_assert (device_role_event);
+
+    srconf_get_data_with_default(PRESENTATION_ACTIVE_SETTING, CFGT_STRING, &active ,DEFAULT_PRESENTATION_ACTIVE_SETTING_NAME , PRESENTATION_PATH);
+    if (active)
+    {
+	gchar *key = g_strconcat (active, "/", device_role_event, NULL);    
+	srconf_get_data_with_default (key, CFGT_STRING, &chunk ,DEFAULT_PRESENTATION_ACTIVE_SETTING_NAME , PRESENTATION_PATH);   
+	g_free (key);	
+    }
+
+    return chunk;
+}
