@@ -213,8 +213,9 @@ src_present_real_sro_for_speech (SRObject *obj, gchar *reason)
 
     src_enable_format_string = TRUE; 
     sro_get_role (obj, &role, SR_INDEX_CONTAINER);
-    if (strcmp (reason, "object:text-changed:insert") == 0
-	    && (role == SR_ROLE_TEXT_SL || role == SR_ROLE_TEXT_ML || role == SR_ROLE_TERMINAL))
+
+    if (strcmp (reason, "object:text-changed:insert") == 0 && role != SR_ROLE_TERMINAL
+	    && (role == SR_ROLE_TEXT_SL || role == SR_ROLE_TEXT_ML))
     {
 	gchar *diff;
 	if (sro_text_get_difference (obj, &diff, SR_INDEX_CONTAINER))
@@ -265,7 +266,12 @@ src_present_real_sro_for_speech (SRObject *obj, gchar *reason)
 			strcmp (reason , "focus:") == 0)
 		    src_speech_send_chunk (txt, SRC_SPEECH_PRIORITY_IDLE, FALSE);
 		else
-		    src_speech_send_chunk (txt, SRC_SPEECH_PRIORITY_IDLE, TRUE);
+		{
+		    gboolean interrupt = TRUE;
+		    if (strcmp (reason, "object:text-changed:insert") == 0 && role == SR_ROLE_TERMINAL)
+			interrupt = FALSE;
+		    src_speech_send_chunk (txt, SRC_SPEECH_PRIORITY_IDLE, interrupt);
+		}
 	    }
 	    else if (srl_is_object_watched (obj))
 	    {
