@@ -29,10 +29,12 @@
 #include "srintl.h"
 #include <libxml/parser.h>
 
+extern gint src_caret_position;
 
 typedef enum
 {
     SRC_PRESENTATION_NAME,
+    SRC_PRESENTATION_DESCRIPTION,
     SRC_PRESENTATION_ROLE,
     SRC_PRESENTATION_ACCELERATOR,
     SRC_PRESENTATION_SHORTCUT,
@@ -61,6 +63,9 @@ typedef enum
     SRC_PRESENTATION_STATE_MINIMIZED,
     SRC_PRESENTATION_CHILDREN_COUNT,
     SRC_PRESENTATION_VALUE_CRT,
+    SRC_PRESENTATION_COLUMN_HEADER,
+    SRC_PRESENTATION_ROW_HEADER,
+    SRC_PRESENTATION_CELL,
     SRC_PRESENTATION_NONE
 }SRCPresentationItemType;
 
@@ -93,6 +98,20 @@ typedef struct _SRCPresentationItem
     gpointer			voice;
 }SRCPresentationItem;
 
+static const gchar *src_pres_lang = NULL;
+
+static const gchar*
+srcp_get_lang (void)
+{
+    if (!src_pres_lang)
+    {
+	src_pres_lang = g_getenv ("LANG");
+	if (!src_pres_lang)
+	    src_pres_lang = "C";
+    }
+    return src_pres_lang;
+}
+
 static SRCPresentationItem*
 src_presentation_item_new ()
 {
@@ -179,6 +198,7 @@ src_presentation_get_item_type (const gchar *item)
     }src_presentation_type_name [] = 
 	{
 	    {SRC_PRESENTATION_NAME,		"name"},
+	    {SRC_PRESENTATION_DESCRIPTION,	"description"},
 	    {SRC_PRESENTATION_ROLE,		"role"},
 	    {SRC_PRESENTATION_ACCELERATOR,	"accelerator"},
 	    {SRC_PRESENTATION_SHORTCUT,		"shortcut"},
@@ -206,7 +226,10 @@ src_presentation_get_item_type (const gchar *item)
 	    {SRC_PRESENTATION_STATE_COLLAPSED,	"state:collapsed"},
 	    {SRC_PRESENTATION_STATE_MINIMIZED,	"state:minimized"},
 	    {SRC_PRESENTATION_CHILDREN_COUNT,	"childcount"},
-	    {SRC_PRESENTATION_VALUE_CRT,	"value:crt"}
+	    {SRC_PRESENTATION_VALUE_CRT,	"value:crt"},
+	    {SRC_PRESENTATION_COLUMN_HEADER,	"column_header"},
+	    {SRC_PRESENTATION_ROW_HEADER,       "row_header"},
+	    {SRC_PRESENTATION_CELL,             "cell"}
 	};
     gint i;
 
@@ -262,13 +285,43 @@ src_presentation_get_presentation_item_from_dots (xmlNode *node,
     return item;
 }
 
+static const xmlNode*
+src_presentation_get_literal_node_lang (xmlNode *start,
+					xmlChar *lang)
+{
+    xmlNode *crt;
+    xmlNode *c = NULL, *partial = NULL;
+
+    sru_assert (start && lang);
+
+    for (crt = start; crt && xmlStrcmp (crt->name, "literal") == 0; crt= crt->next)
+    {
+	xmlChar *nlang = xmlNodeGetLang (crt);
+	if (nlang && xmlStrcmp (lang, nlang) == 0) /* perfect match */
+	{
+	    xmlFree (nlang);
+	    return crt;
+	}
+	if (nlang && xmlStrncmp (lang, nlang, 2) == 0) /* partial match */
+	    partial = crt;
+	if (!nlang && !c)
+	    c = crt;
+	xmlFree (nlang);
+    }
+
+    return partial ? partial : c;
+}
+
+
 static gchar*
 src_presentation_get_from_true_false (xmlNode *node)
 {
-    xmlNode *crt;
-    for (crt = node->xmlChildrenNode; crt; crt= crt->next)
-    	if (xmlStrcmp (crt->name, "literal") == 0)
-	    return g_strdup (xmlNodeGetContent (crt));
+    const xmlNode *crt = NULL;
+
+    if (node && node->xmlChildrenNode)
+	crt = src_presentation_get_literal_node_lang (node->xmlChildrenNode, (xmlChar*)srcp_get_lang ());
+    if (crt)
+	return xmlNodeGetContent ((xmlNode*)crt);
     return NULL;
 }
 
@@ -420,8 +473,15 @@ src_presentation_get_presentation_items_from_node (xmlNode *node,
     for (crt = node->xmlChildrenNode; crt; crt = crt->next)
     {
 	SRCPresentationItem *item = NULL;
+	xmlNode *ant;
 	if (xmlStrcmp (crt->name, "literal") == 0)
-	    item = src_presentation_get_presentation_item_from_literal (crt, device);
+	{
+	    const xmlNode *literal = src_presentation_get_literal_node_lang (crt, (xmlChar*)srcp_get_lang ());
+	    if (literal)
+		item = src_presentation_get_presentation_item_from_literal ((xmlNode*)literal, device);
+	    for (ant = crt ; ant && xmlStrcmp (ant->name, "literal") == 0; ant = ant->next)
+		crt = ant;
+	}
 	else if (xmlStrcmp (crt->name, "brdots") == 0)
 	    item = src_presentation_get_presentation_item_from_dots (crt, device);
 	else if (xmlStrcmp (crt->name, "conditional") == 0)
@@ -745,83 +805,83 @@ static struct
     gchar 	*speech_short;
 }src_presentation_role_pres[] =
     {
-	{"unknown",			"UNK",	"UNKNOWN",		"Unknown",		},
-	{"invalid",			"???",	"INVALID",		"Invalid", 		},
-        {"alert", 			"ALR",	"ALERT",		"Alert",		},
-	{"canvas",			"CNV",	"CANVAS",		"Canvas",		},
-        {"check-box",			"CHK",	"CHECK_BOX",		"Check box",		},
-	{"check-menu-item",		"MIT",	"MENU ITEM",		"Check Menu item",	},
-	{"color-chooser",		"CCH",	"COLOR CHOOSER",	"Color chooser",	},
-        {"column-header",		"CHD",	"COLUMN HEADER",	"Column header",	},
-	{"combo-box",			"CBO",	"COMBO BOX",		"Combo box",		},
-        {"desktop-icon",		"DIC",	"DESKTOP ICON",		"Desktop icon",		},
-	{"desktop-frame",		"DFR",	"DESKTOP FRAME",	"Desktop frame",	},
-        {"dialog",			"DLG",	"DIALOG",		"Dialog",		},
-	{"directory-pane",		"DIP",	"DIRECTORY PANE",	"Directory pane",	},
-        {"file-chooser",		"FCH",	"FILE CHOOSER",		"File chooser",		},
-	{"filler",			"FLR",	"FILLER",		"Filler",		},
-        {"frame",			"FRM",	"FRAME",		"Frame",		},
-        {"glass-pane",			"GPN",	"GLASS PANE",		"Glass pane",		},
-	{"HTML-container",		"HTM",	"HTML CONTAINER",	"H T M L container",	},
-	{"hyper link",			"LNK",	"LINK",			"link",		},
-        {"icon",			"ICO",	"ICON",			"Icon",			},
-        {"internal-frame",		"IFR",	"INTERNAL FRAME",	"Internal frame",	},
-	{"label",			"LBL",	"LABEL",		"Label",		},
-        {"layered-pane",		"LPN",	"LAYERED PANE",		"Layered pane",		},
-	{"link",			"LNK",	"LINK",			"link",			},
-	{"list",			"LST",	"LIST",			"List",			},
-        {"list-item",			"LIT",	"LIST ITEM",		"List item",		},
-	{"menu",			"MNU",	"MENU",			"Menu",			},
-        {"menu-bar",			"MBR",	"MENU BAR",		"Menu bar",		},
-	{"menu-item",			"MIT",	"MENU ITEM",		"Menu item",		},
-        {"option-pane",			"OPN",	"OPTION PANE",		"Option pane",		},
-        {"page-tab",			"PGT",	"PAGE TAB",		"Page tab",		},
-	{"page-tab-list",		"PTL",	"PAGE TAB LIST",	"Page tab list",	},
-        {"panel",			"PNL",	"PANEL",		"Panel",		},
-        {"password-text",		"PWD",	"PASSWORD TEXT",	"Password text",	},
-	{"popup-menu",			"PMN",	"POPUP MENU",		"Popup menu",		},
-        {"progress-bar",		"PRG",	"PROGRESS BAR",		"Progress bar",		},
-	{"push-button",			"PBT",	"PUSH BUTTON",		"Push button",		},
-        {"radio-button",		"RAD",	"RADIO BUTTON",		"Radio button",		},
-	{"radio-menu-item",		"MIT",	"MENU ITEM",		"Radio Menu item",	},
-        {"root-pane",			"RPN",	"ROOT PANE",		"Root pane",		},
-	{"row-header",			"RHD",	"ROW HEADER",		"Row header",		},
-        {"scroll-bar",			"SCR",	"SCROLL BAR",		"Scroll bar",		},
-	{"scroll-pane",			"SPN",	"SCROLL PANE",		"Scroll pane",		},
-        {"separator",			"SEP",	"SEPARATOR",		"Separator",		},
-	{"slider",			"SLD",	"SLIDER",		"Slider",		},
-        {"split-pane",			"SPP",	"SPLIT PANE",		"Split pane",		},
-	{"status-bar",			"STA",	"STATUS BAR",		"Status bar",		},
-        {"table",			"TAB",	"TABLE",		"Table",		},
-	{"table-cell",			"CEL",	"TABLE CELL",		"Table cell",		},
-        {"table-column-header",		"TCH",	"TABLE COLUMN HEADER",	"Table column header",	},
-	{"table-row-header",		"TRH",	"TABLE ROW HEADER",	"Table row header",	},
-        {"multi-line-text",		"TXT",	"MULTI LINE TEXT",	"Multi Line Text",	},
-        {"single-line-text",		"TXT",	"SINGLE LINE TEXT",	"Single Line Text",	},
-	{"toggle-button",		"TOG",	"TOGGLE BUTTON",	"Toggle button",	},
-        {"tool-bar",			"TOL",	"TOOL BAR",		"Tool bar",		},
-	{"tool-tip",			"TIP",	"TOOL TIP",		"Tool tip",		},
-        {"tree",			"TRE",	"TREE",			"Tree",			},
-        {"tree-item",			"TRI",	"TREE ITEM",		"Tree item",		},
-	{"tree-table",			"TRT",	"TREE TABLE",		"Tree table",		},
-	{"viewport",			"VWP",	"VIEWPORT",		"Viewport",		},
-	{"window",			"WND",	"WINDOW",		"Window",		},
-	{"accelerator-label",		"ACC",	"ACCELERATOR LABEL",	"Accelerator label",	},
-        {"animation",			"ANI",	"ANIMATION",		"Animation",		},
-	{"arrow",			"ARR",	"ARROW",		"Arrow",		},
-        {"calendar",			"CAL",	"CALENDAR",		"Calendar",		},
-	{"date-editor",			"DAT",	"DATE EDITOR",		"Date editor",		},
-        {"dial",			"DIL",	"DIAL",			"Dial",			},
-	{"drawing-area",		"DRW",	"DRAWING AREA",		"Drawing area",		},
-        {"font-chooser",		"FNT",	"FONT CHOOSER",		"Font chooser",		},
-	{"image",			"IMG",	"IMAGE",		"Image",		},
-        {"spin-button",			"SPN",	"SPIN BUTTON",		"Spin button",		},
-	{"terminal",			"TRM",	"TERMINAL",		"Terminal",		},
-        {"extended",			"EXT",	"EXTENDED",		"Extended",		},
-	{"table-line",			"TLI",	"TABLE LINE",		"Table line",		},
-        {"table-columns-header",	"TCH","TABLE COLUMNS HEADER",	"Table columns header",	},
-	{"title-bar",			"TIT",	"TITLE BAR",		"Title bar",		},
-	{"edit-bar",			"EDB",	"EDIT BAR",		"Edit bar",		}
+	{"unknown",		N_("UNK"),	N_("UNKNOWN"),		N_("Unknown")			},
+	{"invalid",		N_("???"),	N_("INVALID"),		N_("Invalid")			},
+	{"alert",		N_("ALR"),	N_("ALERT"),		N_("Alert")			},
+	{"canvas",		N_("CNV"),	N_("CANVAS"),		N_("Canvas")			},
+	{"check-box",		N_("CHK"),	N_("CHECK_BOX"),	N_("Check box")			},
+	{"check-menu-item",	N_("MIT"),	N_("MENU ITEM"),	N_("Check Menu item")		},
+	{"color-chooser",	N_("CCH"),	N_("COLOR CHOOSER"),	N_("Color chooser")		},
+	{"column-header",	N_("CHD"),	N_("COLUMN HEADER"),	N_("Column header")		},
+	{"combo-box",		N_("CBO"),	N_("COMBO BOX"),	N_("Combo box")			},
+	{"desktop-icon",	N_("DIC"),	N_("DESKTOP ICON"),	N_("Desktop icon")		},
+	{"desktop-frame",	N_("DFR"),	N_("DESKTOP FRAME"),	N_("Desktop frame")		},
+	{"dialog",		N_("DLG"),	N_("DIALOG"),		N_("Dialog")			},
+	{"directory-pane",	N_("DIP"),	N_("DIRECTORY PANE"),	N_("Directory pane")		},
+	{"file-chooser",	N_("FCH"),	N_("FILE CHOOSER"),	N_("File chooser")		},
+	{"filler",		N_("FLR"),	N_("FILLER"),		N_("Filler")			},
+	{"frame",		N_("FRM"),	N_("FRAME"),		N_("Frame")			},
+	{"glass-pane",		N_("GPN"),	N_("GLASS PANE"),	N_("Glass pane")		},
+	{"HTML-container",	N_("HTM"),	N_("HTML CONTAINER"),	N_("H T M L container")		},
+	{"hyper link",		N_("LNK"),	N_("LINK"),		N_("link")			},
+	{"icon",		N_("ICO"),	N_("ICON"),		N_("Icon")			},
+	{"internal-frame",	N_("IFR"),	N_("INTERNAL FRAME"),	N_("Internal frame")		},
+	{"label",		N_("LBL"),	N_("LABEL"),		N_("Label")			},
+	{"layered-pane",	N_("LPN"),	N_("LAYERED PANE"),	N_("Layered pane")		},
+	{"link",		N_("LNK"),	N_("LINK"),		N_("link")			},
+	{"list",		N_("LST"),	N_("LIST"),		N_("List")			},
+	{"list-item",		N_("LIT"),	N_("LIST ITEM"),	N_("List item")			},
+	{"menu",		N_("MNU"),	N_("MENU"),		N_("Menu")			},
+	{"menu-bar",		N_("MBR"),	N_("MENU BAR"),		N_("Menu bar")			},
+	{"menu-item",		N_("MIT"),	N_("MENU ITEM"),	N_("Menu item")			},
+	{"option-pane",		N_("OPN"),	N_("OPTION PANE"),	N_("Option pane")		},
+	{"page-tab",		N_("PGT"),	N_("PAGE TAB"),		N_("Page tab")			},
+	{"page-tab-list",	N_("PTL"),	N_("PAGE TAB LIST"),	N_("Page tab list")		},
+	{"panel",		N_("PNL"),	N_("PANEL"),		N_("Panel")			},
+	{"password-text",	N_("PWD"),	N_("PASSWORD TEXT"),	N_("Password text")		},
+	{"popup-menu",		N_("PMN"),	N_("POPUP MENU"),	N_("Popup menu")		},
+	{"progress-bar",	N_("PRG"),	N_("PROGRESS BAR"),	N_("Progress bar")		},
+	{"push-button",		N_("PBT"),	N_("PUSH BUTTON"),	N_("Push button")		},
+	{"radio-button",	N_("RAD"),	N_("RADIO BUTTON"),	N_("Radio button")		},
+	{"radio-menu-item",	N_("MIT"),	N_("MENU ITEM"),	N_("Radio Menu item")		},
+	{"root-pane",		N_("RPN"),	N_("ROOT PANE"),	N_("Root pane")			},
+	{"row-header",		N_("RHD"),	N_("ROW HEADER"),	N_("Row header")		},
+	{"scroll-bar",		N_("SCR"),	N_("SCROLL BAR"),	N_("Scroll bar")		},
+	{"scroll-pane",		N_("SPN"),	N_("SCROLL PANE"),	N_("Scroll pane")		},
+	{"separator",		N_("SEP"),	N_("SEPARATOR"),	N_("Separator")			},
+	{"slider",		N_("SLD"),	N_("SLIDER"),		N_("Slider")			},
+	{"split-pane",		N_("SPP"),	N_("SPLIT PANE"),	N_("Split pane")		},
+	{"status-bar",		N_("STA"),	N_("STATUS BAR"),	N_("Status bar")		},
+	{"table",		N_("TAB"),	N_("TABLE"),		N_("Table")			},
+	{"table-cell",		N_("CEL"),	N_("TABLE CELL"),	N_("Table cell")		},
+	{"table-column-header",	N_("TCH"),	N_("TABLE COLUMN HEADER"),N_("Table column header")	},
+	{"table-row-header",	N_("TRH"),	N_("TABLE ROW HEADER"),	N_("Table row header")		},
+	{"multi-line-text",	N_("TXT"),	N_("MULTI LINE TEXT"),	N_("Multi Line Text")		},
+	{"single-line-text",	N_("TXT"),	N_("SINGLE LINE TEXT"),	N_("Single Line Text")		},
+	{"toggle-button",	N_("TOG"),	N_("TOGGLE BUTTON"),	N_("Toggle button")		},
+	{"tool-bar",		N_("TOL"),	N_("TOOL BAR"),		N_("Tool bar")			},
+	{"tool-tip",		N_("TIP"),	N_("TOOL TIP"),		N_("Tool tip")			},
+	{"tree",		N_("TRE"),	N_("TREE"),		N_("Tree")			},
+	{"tree-item",		N_("TRI"),	N_("TREE ITEM"),	N_("Tree item")			},
+	{"tree-table",		N_("TRT"),	N_("TREE TABLE"),	N_("Tree table")		},
+	{"viewport",		N_("VWP"),	N_("VIEWPORT"),		N_("Viewport")			},
+	{"window",		N_("WND"),	N_("WINDOW"),		N_("Window")			},
+	{"accelerator-label",	N_("ACC"),	N_("ACCELERATOR LABEL"),N_("Accelerator label")		},
+	{"animation",		N_("ANI"),	N_("ANIMATION"),	N_("Animation")			},
+	{"arrow",		N_("ARR"),	N_("ARROW"),		N_("Arrow")			},
+	{"calendar",		N_("CAL"),	N_("CALENDAR"),		N_("Calendar")			},
+	{"date-editor",		N_("DAT"),	N_("DATE EDITOR"),	N_("Date editor")		},
+	{"dial",		N_("DIL"),	N_("DIAL"),		N_("Dial")			},
+	{"drawing-area",	N_("DRW"),	N_("DRAWING AREA"),	N_("Drawing area")		},
+	{"font-chooser",	N_("FNT"),	N_("FONT CHOOSER"),	N_("Font chooser")		},
+	{"image",		N_("IMG"),	N_("IMAGE"),		N_("Image")			},
+	{"spin-button",		N_("SPN"),	N_("SPIN BUTTON"),	N_("Spin button")		},
+	{"terminal",		N_("TRM"),	N_("TERMINAL"),		N_("Terminal")			},
+	{"extended",		N_("EXT"),	N_("EXTENDED"),		N_("Extended")			},
+	{"table-line",		N_("TLI"),	N_("TABLE LINE"),	N_("Table line")		},
+	{"table-columns-header",N_("TCH"),	N_("TABLE COLUMNS HEADER"),N_("Table columns header")	},
+	{"title-bar",		N_("TIT"),	N_("TITLE BAR"),	N_("Title bar")			},
+	{"edit-bar",		N_("EDB"),	N_("EDIT BAR"),		N_("Edit bar")			}
     };
 
 static gint
@@ -850,7 +910,7 @@ src_presentation_get_role_name_for_braille (const gchar *role)
     index = src_presentation_get_role_index (role);
 
     if (index)
-    return src_presentation_role_pres[index].braille_short;
+	return _(src_presentation_role_pres[index].braille_short);
     else
 	return role;
 }
@@ -865,7 +925,7 @@ src_presentation_get_role_name_for_speech (const gchar *role)
     index = src_presentation_get_role_index (role);
 
     if (index)
-    return src_presentation_role_pres[index].speech_short;
+	return _(src_presentation_role_pres[index].speech_short);
     else
 	return role;
 }
@@ -923,6 +983,17 @@ src_presentation_get_info_from_sro (SRObject *obj,
 		}
 	    }
 	    break;
+	case SRC_PRESENTATION_DESCRIPTION:
+	    {
+		gchar *tmp;
+		rv = sro_get_description (obj, &tmp, SR_INDEX_CONTAINER);
+		if (rv)
+		{	
+		    *((gchar **)info) = g_strdup (tmp);
+		    SR_freeString (tmp);
+		}
+	    }
+	    break;
 	case SRC_PRESENTATION_ROLE:
 	    {
 		gchar *tmp;
@@ -999,7 +1070,6 @@ src_presentation_get_info_from_sro (SRObject *obj,
 		    rv = (state & SR_STATE_MINIMIZED) ? TRUE : FALSE;
 	    }
 	    break;
-
 	case SRC_PRESENTATION_STATE_ENABLED:
 	    {
 		SRState state;
@@ -1007,8 +1077,6 @@ src_presentation_get_info_from_sro (SRObject *obj,
 		    rv = state & SR_STATE_ENABLED;
 	    }
 	    break;
-	
-
 	case SRC_PRESENTATION_CHILDREN_COUNT:
 	    {
 		guint32 cc;
@@ -1042,9 +1110,14 @@ src_presentation_get_info_from_sro (SRObject *obj,
 		
 		if (last_offset == crt_offset)
 		    last_offset = last_offset2;
-		if (abs (crt_offset - last_offset) == 1)
+		if (abs (crt_offset - last_offset) == 1 &&
+		    sro_text_is_same_line (obj, last_offset, SR_INDEX_CONTAINER))
 		    type = SR_TEXT_BOUNDARY_CHAR;
-		else if (sro_text_is_same_line (obj, last_offset, SR_INDEX_CONTAINER))
+		else if (abs (crt_offset - last_offset) == 1 &&
+		         !sro_text_is_same_line (obj, last_offset, SR_INDEX_CONTAINER))
+		    type = SR_TEXT_BOUNDARY_LINE;    
+		else if (sro_text_is_same_line (obj, last_offset, SR_INDEX_CONTAINER) ||
+		         sro_is_word_navigation (obj, crt_offset, last_offset, SR_INDEX_CONTAINER))
 		    type = SR_TEXT_BOUNDARY_WORD;
 		else
 		    type = SR_TEXT_BOUNDARY_LINE;
@@ -1099,22 +1172,68 @@ src_presentation_get_info_from_sro (SRObject *obj,
 		rv = FALSE;
 		if (sro_value_get_crt_val (obj, &val, SR_INDEX_CONTAINER))
 		{
-		    *((gchar **)info) = g_strdup_printf ("%.0f", val);
+		    if (val == (int)val)
+			*((gchar **)info) = g_strdup_printf ("%.0f", val);
+		    else	
+			*((gchar **)info) = g_strdup_printf ("%.2f", val);
 		    rv = TRUE;
 		}
 	    }
 	    break;
+	case SRC_PRESENTATION_COLUMN_HEADER:
+	    {
+		gchar *tmp;
+		rv = sro_get_column_header (obj, &tmp, SR_INDEX_CONTAINER);
+		if (rv)
+		{
+		    *((gchar **)info) = g_strdup (tmp);
+		    SR_freeString (tmp);
+		}
+	    }
+	    break;
+	case SRC_PRESENTATION_ROW_HEADER:
+	    {
+		gchar *tmp;
+		rv = sro_get_row_header (obj, &tmp, SR_INDEX_CONTAINER);
+		if (rv)
+		{
+		    *((gchar **)info) = g_strdup (tmp);
+		    SR_freeString (tmp);
+		}
+	    }
+	    break;    
+	case SRC_PRESENTATION_CELL:
+	    {
+		gchar *tmp;
+		rv = sro_get_cell (obj, &tmp, SR_INDEX_CONTAINER);
+		if (rv)
+		{
+		    *((gchar **)info) = g_strdup (tmp);
+		    SR_freeString (tmp);
+		}
+	    }    
+	    break;
 	default:
 	    fprintf (stderr, "\n%d", info_type);/* removable */
 	    sru_assert_not_reached ();	
 	    break;
     }
+    if (info && *(gchar**)info)
+    {
+	if (!g_utf8_validate (*((gchar**)info), -1, NULL))
+	{
+	    sru_warning ("Invalid info queried (%d) ", info_type);
+	    g_free (*(gchar**)info);
+	    *(gchar**)info = NULL;
+	    rv = FALSE;
+	}
+    }
     return rv;
 }
 
 
 static gchar*
-src_presentation_present_sro_for_device_ (SRObject *obj,
+src_presentation_sro_get_pres_for_device_ (SRObject *obj,
 					  const gchar *device,
 					  SRCPresentationItem **chunk)
 {
@@ -1140,7 +1259,9 @@ src_presentation_present_sro_for_device_ (SRObject *obj,
 			if (info)
 			{
 			    add = src_xml_format ("TEXT", has_voice ? src_speech_get_voice ((gchar *)chunk[i]->voice) : NULL, info);
-			    g_free (info);
+			    if (chunk[i]->info != SRC_PRESENTATION_TEXT_LINE)
+					src_caret_position += strlen (info);
+				g_free (info);
 			}
 		    }
 		}
@@ -1162,7 +1283,7 @@ src_presentation_present_sro_for_device_ (SRObject *obj,
 			      continue;
 			    
 			    tmp = add;
-			    tmp2 = src_presentation_present_sro_for_device_ (child, device, (SRCPresentationItem **)chunk[i]->data1);
+			    tmp2 = src_presentation_sro_get_pres_for_device_ (child, device, (SRCPresentationItem **)chunk[i]->data1);
 			    
 			    tmp3 = NULL;
 			    if (j != cnt - 1 && chunk[i]->data2)
@@ -1197,7 +1318,10 @@ src_presentation_present_sro_for_device_ (SRObject *obj,
 		    {
 			add = src_xml_format ("TEXT", has_voice ? src_speech_get_voice ((gchar *)chunk[i]->voice) : NULL, (gchar *)chunk[i]->data1);
 			if (info)
+			{
+				src_caret_position += strlen (info);
 			    g_free (info);
+			}
 		    }
 		    else
 		    {
@@ -1218,6 +1342,7 @@ src_presentation_present_sro_for_device_ (SRObject *obj,
 		    if (info2)
 		    {
 			add = src_xml_format ("TEXT", has_voice ? src_speech_get_voice ((gchar *)chunk[i]->voice) : NULL, info2);
+			src_caret_position +=  strlen (info2);
 			g_free (info2);
 		    }
 		}
@@ -1232,7 +1357,10 @@ src_presentation_present_sro_for_device_ (SRObject *obj,
 			else
 			    add = src_xml_format ("TEXT", has_voice ? src_speech_get_voice ((gchar *)chunk[i]->voice) : NULL, (gchar *)chunk[i]->data2);
 			if (info)
-			    g_free (info);
+			{
+			   src_caret_position += strlen (info);
+			   g_free (info);
+			}
 		    }
 		    else
 		    {
@@ -1253,6 +1381,7 @@ src_presentation_present_sro_for_device_ (SRObject *obj,
 			if (tmp)
 			{
 			    add = src_xml_make_part ("DOTS", NULL, tmp);
+				src_caret_position += strlen (tmp);
 			    g_free (tmp);
 			}
 		    }
@@ -1266,6 +1395,7 @@ src_presentation_present_sro_for_device_ (SRObject *obj,
 	    gchar *tmp = rv;
 	    if (strcmp (device, "braille") == 0)
 	    {
+		src_caret_position ++;
 		rv = g_strconcat (rv, rv[0] != '\0' ? "<TEXT> </TEXT>" : "", add, NULL); 
 	    }
 	    else
@@ -1283,7 +1413,7 @@ src_presentation_present_sro_for_device_ (SRObject *obj,
 
 
 gchar*
-src_presentation_present_sro_for_magnifier (SRObject *obj, 
+src_presentation_sro_get_pres_for_magnifier (SRObject *obj, 
 					    const gchar *reason)
 {
     gchar *role;
@@ -1333,7 +1463,7 @@ src_presentation_present_sro_for_magnifier (SRObject *obj,
 
 
 gchar*
-src_presentation_present_sro_for_speech (SRObject *obj, 
+src_presentation_sro_get_pres_for_speech (SRObject *obj, 
 					 const gchar *reason)
 {
     SRCPresentationItem **presentation;
@@ -1377,7 +1507,7 @@ src_presentation_present_sro_for_speech (SRObject *obj,
 	}
 	else if (src_presentation_find_chunk_for_speech (role, reason, &presentation))
 	{
-	    presentation_ = src_presentation_present_sro_for_device_ (obj, device, presentation);
+	    presentation_ = src_presentation_sro_get_pres_for_device_ (obj, device, presentation);
 	}
 	SR_freeString (role);    
     }
@@ -1386,7 +1516,7 @@ src_presentation_present_sro_for_speech (SRObject *obj,
 }
 
 gchar*
-src_presentation_present_sro_for_braille (SRObject *obj, 
+src_presentation_sro_get_pres_for_braille (SRObject *obj, 
 					  const gchar *reason)
 {
     SRCPresentationItem  **presentation;
@@ -1400,7 +1530,7 @@ src_presentation_present_sro_for_braille (SRObject *obj,
     {
 	if (src_presentation_find_chunk_for_braille (role, reason, &presentation))
 	{
-	    presentation_ = src_presentation_present_sro_for_device_ (obj, device, presentation);
+	    presentation_ = src_presentation_sro_get_pres_for_device_ (obj, device, presentation);
 	}
 	SR_freeString (role);    
     }
@@ -1410,7 +1540,7 @@ src_presentation_present_sro_for_braille (SRObject *obj,
 
 
 gchar *
-src_presentation_present_sro_for_device (SRObject *obj, 
+src_presentation_sro_get_pres_for_device (SRObject *obj, 
 					 const gchar *reason, 
 					 const gchar *device)
 {
@@ -1419,11 +1549,11 @@ src_presentation_present_sro_for_device (SRObject *obj,
     sru_assert (obj && device && reason);
 
     if (strcmp (device, "magnifier") == 0)
-	presentation_ = src_presentation_present_sro_for_magnifier (obj, reason);
+	presentation_ = src_presentation_sro_get_pres_for_magnifier (obj, reason);
     else  if (strcmp (device, "speech") == 0)
-	presentation_ = src_presentation_present_sro_for_speech (obj, reason);
+	presentation_ = src_presentation_sro_get_pres_for_speech (obj, reason);
     else if (strcmp (device, "braille") == 0)
-	presentation_ = src_presentation_present_sro_for_braille (obj, reason);
+	presentation_ = src_presentation_sro_get_pres_for_braille (obj, reason);
     else
 	sru_assert_not_reached ();
 
@@ -1440,5 +1570,8 @@ src_pres_get_role_name_for_speech (const gchar *role)
 
     index = src_presentation_get_role_index (role);
 
-    return src_presentation_role_pres[index].speech_short;
+    if (index)
+	return _(src_presentation_role_pres[index].speech_short);
+    else
+	return role;
 }
