@@ -27,7 +27,7 @@
 
 #include "srmain.h"
 #include "srspc.h"
-#include "../speech/libsrs/srsxmlapi.h"
+#include "srs-xml.h"
 #include "SREvent.h"
 #include "srpres.h"
 #include "libsrconf.h"
@@ -37,6 +37,7 @@
 #include "sys/stat.h"
 #include "fcntl.h"
 #include "unistd.h"
+#include "src-speech-in.h"
 
 #define STR(X) (X == NULL? "" : X)
 
@@ -72,99 +73,42 @@ typedef struct
     gchar *rate;
     gchar *pitch;
     gchar *volume;
-
 }SRCVoice;
 
+typedef struct _SRCSOut
+{
+    gchar *chunk;
+    SRCSpeechPriority priority;
+    gboolean preempt;
+    gchar *ido;
+}SRCSOut;
+
 static GPtrArray     	*src_voice_params = NULL;
 SRCSpeech 	 	*src_speech = NULL;
 static SRCVoices 	*src_voices = NULL;
 
-static gchar**
-src_speech_get_engine_drivers ()
+static gboolean src_speech_idle = TRUE;
+static GSList *srcs_outs = NULL;
+static SRCSOut *last_out = NULL;
+#ifdef SRS_NO_MARKERS_SUPPORTED
+static gboolean src_speech_callback = TRUE;
+static GSList  *src_speech_unspoken = NULL;
+#endif
+
+
+static SRCSOut*
+srcs_out_new ()
 {
-    gchar **rv;
-    gint  count;
-    gint  index;
-    GSList *drivers = NULL;
-    GSList *tmp     = NULL;
-    
-    sru_assert (src_speech_initialized);
-
-    srconf_get_data_with_default (SPEECH_ENGINE_DRIVERS, CFGT_LIST, 
-				&drivers, NULL, SPEECH_DRIVERS_SECTION);
-
-    if (!drivers)
-	return NULL;
-	
-    count = g_slist_length (drivers);
-    
-    rv = g_new (gchar*, count + 1);
-        
-    index = 0;
-    
-    for (tmp = drivers; tmp ; tmp = tmp->next)
-    {
-	if (tmp->data)
-	{
-	    rv[index] = g_strdup (tmp->data);
-	    /*fprintf (stderr, "\nENGINE DRIVER[%d]=%s", index, (gchar*)tmp->data);*/
-	    index++;
-	}
-	g_free (tmp->data);
-    }
-    
-    rv[index] = NULL;
-    
-    g_slist_free (drivers);
-    
-    return rv;
+    return g_new0 (SRCSOut, 1);
 }
 
-static gchar**
-src_speech_get_engine_voices ()
+static void
+srcs_out_terminate (SRCSOut *out)
 {
-    gint index2;
-    gint iter;
-    gint count = -1;
-    gchar **rv;
-    GSList *voices  = NULL;
-    GSList *tmp2    = NULL;
-
-    sru_assert (src_speech_initialized);
-    sru_assert (src_voices->engine_drivers);
-    
-
-    srconf_get_data_with_default (SPEECH_VOICE_COUNT, CFGT_INT, 
-				&count, &count, SPEECH_DRIVERS_SECTION);
-
-    if (count <= 0)
-	return NULL;
-
-    rv = g_new (gchar*, count + 1);
-    
-    index2 = 0;
-    iter = 0;
-
-    for (iter = 0; src_voices->engine_drivers[iter] ; iter++)
-    {
-	srconf_get_data_with_default (src_voices->engine_drivers[iter], CFGT_LIST, 
-				      &voices, NULL, SPEECH_DRIVERS_SECTION);
-	for (tmp2 = voices; tmp2 ; tmp2 = tmp2->next)
-	{
-	    if (tmp2->data && ((gchar*)tmp2->data)[0])
-	    {
-		rv[index2] = g_strdup (tmp2->data);
-/*		fprintf (stderr, "\nENGINE VOICE[%d]=%s", index2, rv[index2]);*/
-		index2++;
-	    }
-	    g_free (tmp2->data);
-	}
-	g_slist_free (voices);
-	voices = NULL;
-    }
-    rv[index2] = NULL;
-
-    return rv;
+    sru_assert (out);
+    g_free (out->chunk);
+    g_free (out->ido);
+    g_free (out);
 }
 
 
@@ -526,7 +470,10 @@ src_speech_send (gchar *spcoutput)
     if (busy)
 	return FALSE;
     busy = TRUE;
-    srs_xml_output (spcoutput, strlen(spcoutput)); 
+    if (g_utf8_validate (spcoutput, -1, NULL))
+	srs_output (spcoutput, strlen(spcoutput)); 
+    else
+	    sru_warning ("speech output: invalid UTF-8 received");
     rv = TRUE;
     
 /*     fprintf (stderr, "\n%s", spcoutput); */
@@ -540,10 +487,9 @@ src_speech_send (gchar *spcoutput)
     return rv;
 }
 
-
 static gboolean
 src_speech_create_voice (gchar *voice,
-			 gchar *driver_,
+			 gchar *driver_, 
 			 gchar *name_,
 			 gchar *volume_,
 			 gchar *rate_,
@@ -551,18 +497,31 @@ src_speech_create_voice (gchar *voice,
 {
     gchar *voice_;
     gchar *name, *volume, *rate, *pitch;
-
+    gchar *tts_drv, *tts_voice;
     gboolean rv = FALSE;
 
     sru_assert (src_speech_initialized);
     sru_assert (voice);
-
     name = volume = rate = pitch = NULL;
     if (name_ && driver_)
     {
 	sru_assert (src_speech_valid_engine_voice (driver_, name_));
 	name = g_strconcat (" TTSVoice=\"", name_, "\"", NULL);
     }
+    tts_drv = tts_voice = NULL;
+    if (name_)
+    {
+	gchar *startd = strstr (name_, "-");
+	gchar *startv = strstr (name_, " ");
+	if (startd && startv)
+	{
+	    gchar *tmp = g_strndup (startv + 1, startd - startv - 2);
+	    tts_voice = g_strconcat (" TTSVoice=\"", tmp, "\"", NULL);
+	    g_free (tmp);
+	}
+	if (startd)
+	    tts_drv = g_strconcat (" TTSEngine=\"", startd + 2, "\"", NULL);
+    }
     if (rate_)
 	rate = g_strconcat (" rate=\"", rate_, "\"", NULL);
     if (volume_)
@@ -570,31 +529,33 @@ src_speech_create_voice (gchar *voice,
     if (pitch_)
 	pitch = g_strconcat (" pitch=\"", pitch_, "\"", NULL);
 
-    voice_ = g_strconcat (	"<SRSOUT>"
-				    "<VOICE"
-					" ID=\"", voice, "\""
-					" TTSEngine=\"gnome-speech\"",
-					/* " priority=\"0\"" */
-					/* " preempt=\"yes\"", */
-					name ? name : "",
-					rate ? rate : "",
-					pitch ? pitch : "",
-					volume ? volume : "",
-				    "></VOICE>"
-				"</SRSOUT>",
-				NULL);
-
+    voice_ = g_strconcat ("<SRSOUT>"
+		    	    "<VOICE"
+			    " ID=\"", voice, "\"",
+			    /*" TTSEngine=\"gnome-speech\"",*/
+			    /* " priority=\"0\"" */
+			    /* " preempt=\"yes\"", */
+			    /*name ? name : "",*/
+			    tts_drv ? tts_drv : "",
+			    tts_voice ? tts_voice : "",
+			    rate ? rate : "",
+			    pitch ? pitch : "",
+			    volume ? volume : "",
+			    "></VOICE>"
+			    "</SRSOUT>",
+			    NULL);
     if (voice_)
 	rv = src_speech_send (voice_);
     if (!rv)
-	sru_warning (_("could not create/modify voice \"%s\""), voice);
+        sru_warning (_("could not create/modify voice \"%s\""), voice);
     g_free (voice_);
-    
     g_free (name);
     g_free (rate);
     g_free (pitch);
     g_free (volume);
-    
+    g_free (tts_drv);
+    g_free (tts_voice);
+
     return rv;
 }
 
@@ -1160,6 +1121,29 @@ void
 src_speech_shutup ()
 {
     sru_assert (src_speech_initialized);
+#ifdef SRS_NO_MARKERS_SUPPORTED
+    {
+	GSList *crt, *tmp = src_speech_unspoken;
+	src_speech_unspoken = NULL;
+	for (crt = tmp; crt; crt = crt->next)
+	    srcs_out_terminate (crt->data);
+	g_slist_free (tmp);
+    }
+#endif
+    if (last_out)
+    {
+        src_speech_idle = TRUE;
+        srcs_out_terminate (last_out);
+	last_out = NULL;
+    }
+    if (srcs_outs)
+    {
+	GSList *crt, *tmp = srcs_outs;
+	srcs_outs = NULL;
+	for (crt = tmp; crt; crt = crt->next)
+	    srcs_out_terminate (crt->data);
+	g_slist_free (tmp);
+    }
 
     src_speech_send ("<SRSOUT><SHUTUP/></SRSOUT>");
 }
@@ -1183,22 +1167,96 @@ src_speech_resume ()
 }
 
 
-gboolean
-src_speech_send_chunk (gchar *chunk, SRCSpeechPriority priority, gboolean preempt)
+static gboolean
+src_speech_send_from_queue ()
 {
+    gboolean rv;
     gchar *output;
-    gboolean rv = FALSE;
 
-    sru_assert (src_speech_initialized);
-    sru_assert (chunk && chunk[0]);
-    output = g_strdup_printf ("<SRSOUT priority=\"%d\" preempt=\"%s\">%s</SRSOUT>",
-			priority, preempt ? "true" : "false", chunk);
+    sru_assert (srcs_outs);
+#ifdef SRS_NO_MARKERS_SUPPORTED
+    if (!src_speech_callback)
+    {
+	sru_assert (g_slist_length (srcs_outs) == 1);
+	if (last_out)
+	{
+	    src_speech_unspoken = g_slist_append (src_speech_unspoken, last_out);
+	    last_out = NULL;
+	    src_speech_idle = TRUE;
+	}
+    }
+#endif
+
+    sru_assert (last_out == NULL);
+    last_out = srcs_outs->data;
+    srcs_outs = g_slist_remove_link (srcs_outs, srcs_outs);
+    sru_assert (src_speech_idle);
+    output = g_strdup_printf ("<SRSOUT markers=\"out-started:out-ended\" id=\"%xp\">%s</SRSOUT>", (unsigned int)last_out->chunk, last_out->chunk);
+    src_speech_idle = FALSE;
     rv = src_speech_send (output);
     g_free (output);
 
     return rv;
 }
 
+static gboolean
+srcs_out_can_shutup_out (SRCSOut *crt,
+			 SRCSOut *new_)
+{
+    sru_assert (crt && new_);
+    return  (crt->priority < new_->priority ||
+		(crt->priority == new_->priority && crt->preempt));
+}
+
+gboolean
+src_speech_send_chunk (gchar *chunk,
+		       SRCSpeechPriority priority,
+		       gboolean preempt)
+{
+    SRCSOut *out;
+    GSList *crt, *tmp;
+	
+    sru_assert (src_speech_initialized);
+    sru_assert (chunk && chunk[0]);
+		
+    out = srcs_out_new ();
+    out->chunk = g_strdup (chunk);
+    out->ido = g_strdup_printf ("%xp",(unsigned int) out->chunk);
+    out->priority = priority;
+    out->preempt = preempt;
+	
+    for (crt = srcs_outs; crt; crt = crt->next)
+    {
+        if (srcs_out_can_shutup_out (crt->data, out))
+	    break;
+    }
+    tmp = crt;
+    for (; crt; crt = crt->next)
+	srcs_out_terminate (crt->data);
+    if (tmp)
+    {
+        g_slist_free (tmp->next);
+	tmp->next = NULL;
+	srcs_outs = g_slist_remove_link (srcs_outs, tmp);
+    }
+										        
+    if (last_out && srcs_out_can_shutup_out (last_out, out))
+    {
+	src_speech_shutup ();
+    }
+
+    srcs_outs = g_slist_append (srcs_outs, out);
+#ifdef SRS_NO_MARKERS_SUPPORTED
+    if (!src_speech_callback)
+	src_speech_send_from_queue ();
+    else
+#endif
+    if (src_speech_idle)
+        src_speech_send_from_queue ();
+
+    return TRUE;
+}
+
 static gboolean
 src_dictionary_hash_init ()
 {
@@ -1462,6 +1520,9 @@ src_speech_init ()
 {
     sru_assert (!src_speech_initialized);
     
+    src_speech_idle = TRUE;
+    srcs_outs = NULL;
+    last_out = NULL;
     src_voice_params = g_ptr_array_new ();
     
     src_voices = src_speech_src_voices_new ();
@@ -1469,12 +1530,15 @@ src_speech_init ()
     
     src_speech_set_default_values (src_speech);
     
-    src_speech_initialized = srs_xml_init (NULL);
+    srcs_in_init ();
+    src_speech_initialized = srs_init (srcs_in_parse_input);
+#ifdef SRS_NO_MARKERS_SUPPORTED
+    src_speech_callback = TRUE;
+    src_speech_unspoken = NULL;
+#endif
 
     if (src_speech_initialized)
     {
-	src_voices->engine_drivers= src_speech_get_engine_drivers ();
-	src_voices->engine_voices = src_speech_get_engine_voices  ();
 	src_voices->voices 	  = src_speech_get_voices ();
     }
     
@@ -1526,7 +1590,12 @@ src_speech_terminate ()
 {
     sru_assert (src_speech_initialized);
 
-    srs_xml_terminate();
+    src_speech_shutup ();
+#ifdef SRS_NO_MARKERS_SUPPORTED
+    sru_assert (src_speech_unspoken == NULL);
+#endif
+    srcs_in_terminate ();
+    srs_terminate();
     src_speech_voices_terminate ();
     src_dictionary_terminate ();
     src_speech_initialized = FALSE;    
@@ -1791,7 +1860,6 @@ src_speech_get_text_voice_attributes (gchar *voice,
 }
 
 
-#define SRC_SPEECH_PITCH_STEP 5
 gboolean
 src_speech_modify_pitch (SRCModifyMode mode)
 {
@@ -1825,7 +1893,7 @@ src_speech_modify_pitch (SRCModifyMode mode)
 		    pitch = atoi (pitch_);
 		    g_free (pitch_);
 		    pitch += (mode == SRC_MODIF_DECREASE) ? 
-			    -SRC_SPEECH_PITCH_STEP : SRC_SPEECH_PITCH_STEP;
+			    -SPEECH_PITCH_STEP : SPEECH_PITCH_STEP;
 		}	
 	    }
 
@@ -1840,7 +1908,6 @@ src_speech_modify_pitch (SRCModifyMode mode)
 }
 
 
-#define SRC_SPEECH_RATE_STEP 5
 gboolean
 src_speech_modify_rate (SRCModifyMode mode)
 {
@@ -1874,7 +1941,7 @@ src_speech_modify_rate (SRCModifyMode mode)
 		    rate = atoi (rate_);
 		    g_free (rate_);
 		    rate += (mode == SRC_MODIF_DECREASE) ? 
-			    -SRC_SPEECH_RATE_STEP : SRC_SPEECH_RATE_STEP;
+			    -SPEECH_RATE_STEP : SPEECH_RATE_STEP;
 		}	
 	    }
 
@@ -1890,7 +1957,6 @@ src_speech_modify_rate (SRCModifyMode mode)
 }
 
 
-#define SRC_SPEECH_VOLUME_STEP 5
 gboolean
 src_speech_modify_volume (SRCModifyMode mode)
 {
@@ -1924,7 +1990,7 @@ src_speech_modify_volume (SRCModifyMode mode)
 		    volume = atoi (volume_);
 		    g_free (volume_);
 		    volume += (mode == SRC_MODIF_DECREASE) ? 
-			    -SRC_SPEECH_VOLUME_STEP : SRC_SPEECH_VOLUME_STEP;
+			    -SPEECH_VOLUME_STEP : SPEECH_VOLUME_STEP;
 		}
 	    }
 
@@ -2098,3 +2164,172 @@ src_speech_get_cursors_echo_type ()
     return src_speech->cursors_echo_type;
 }
 /*FIXME end*/
+
+gboolean
+src_speech_set_idle (gchar *ido)
+{
+    sru_assert (ido);
+
+    if (last_out && strcmp (ido, last_out->ido) == 0)
+    {
+        src_speech_idle = TRUE;
+        srcs_out_terminate (last_out);
+	last_out = NULL;
+	if (srcs_outs)
+	    src_speech_send_from_queue ();
+    }
+    return TRUE;
+}
+
+gboolean
+src_speech_add_created_voice (gchar *name
+#ifdef SRS_NO_MARKERS_SUPPORTED
+    			    , gboolean has_callback
+#endif
+			    )
+{
+    
+#ifdef SRS_NO_MARKERS_SUPPORTED
+    if (src_speech_callback)
+    {
+	src_speech_callback = has_callback;
+	if (!src_speech_callback)
+	    sru_warning ("You are using at least one voice witout markers");
+    }
+#endif
+
+    return TRUE;
+}
+
+SRSDriver*
+srs_driver_new ()
+{
+    return g_new0 (SRSDriver, 1);
+}
+
+static void
+srs_driver_terminate (SRSDriver *driver)
+{
+    sru_assert (driver);
+
+    g_free (driver->name);
+    g_strfreev (driver->voices);
+    g_free (driver);
+}
+
+static void
+srs_drivers_terminate (GPtrArray *drivers)
+{
+    gint i;
+
+    sru_assert (drivers);
+    for (i = 0; i < drivers->len - 1; i++)
+	srs_driver_terminate (g_ptr_array_index (drivers, i));
+    g_ptr_array_free (drivers, TRUE);
+}
+
+static gchar*
+srs_normalize_driver_name (gchar *driver_name)
+{
+    gchar *new_driver_name;
+
+    sru_assert (driver_name);
+    new_driver_name = g_strdup (driver_name);
+    new_driver_name = g_strdelimit (new_driver_name, "|> <", '_');
+
+    return new_driver_name;
+}
+
+static gboolean
+srs_write_driver_info_to_gconf (gchar *driver_name,
+			       GSList *voices)
+{
+    gchar *name;
+    sru_assert (driver_name);
+    sru_assert (voices);
+
+    name = srs_normalize_driver_name (driver_name);
+    srconf_set_data (name, CFGT_LIST, voices, SPEECH_DRIVERS_SECTION);
+    g_free (name);
+
+    return TRUE;
+}
+
+static gboolean
+srs_write_drivers_to_gconf (GSList *drivers)
+{
+    sru_assert (drivers);
+
+    srconf_set_data (SPEECH_ENGINE_DRIVERS, CFGT_LIST, drivers,
+			    SPEECH_DRIVERS_SECTION);
+    return TRUE;
+}
+
+static gboolean
+srs_write_voice_count_to_gconf (gint voice_cnt)
+{
+    srconf_set_data (SPEECH_VOICE_COUNT, CFGT_INT, &voice_cnt,
+			SPEECH_DRIVERS_SECTION);
+    return TRUE;
+}
+
+typedef void (*FreeType) (gpointer data);
+
+static void
+srs_slist_free (GSList *list, FreeType func)
+{
+     GSList *crt;
+     sru_assert (list);
+     sru_assert (func);
+     for (crt = list; crt; crt = crt->next)
+        func (crt->data);
+    g_slist_free (list);
+}
+
+gboolean
+src_speech_process_drivers (GPtrArray *drivers)
+{
+    gint i, j, voice_cnt = 0;
+    GSList *drivers_ = NULL;
+    GPtrArray *driver_names;
+    GPtrArray *voice_names;
+
+    driver_names = g_ptr_array_new ();
+    voice_names = g_ptr_array_new ();
+    for (i = 0; i < drivers->len; i++)
+    {
+	GSList *temp = NULL;
+	SRSDriver *driver;
+
+	driver = g_ptr_array_index (drivers, i);
+	sru_assert (driver);
+	drivers_ = g_slist_append (drivers_, 
+			       srs_normalize_driver_name (driver->name));
+	 g_ptr_array_add (driver_names, srs_normalize_driver_name (driver->name));
+        /* fprintf (stderr, "\nDRIVER:%s",  srs_normalize_driver_name (driver->name)); */
+        for (j = 0; driver->voices[j]; j++)
+	{
+	    gchar *voice_name;
+	    voice_name = g_strdup_printf ("V%d %s - %s", j, driver->voices[j],
+					    	      driver->name);
+	    temp = g_slist_append (temp, voice_name);
+	    g_ptr_array_add (voice_names, g_strdup (voice_name));
+	    /* fprintf (stderr, "\n\tVOICE:%s", voice_name); */
+	    voice_cnt++;
+	}
+	srs_write_driver_info_to_gconf (driver->name, temp);
+	srs_slist_free (temp, g_free);
+    }
+    srs_write_voice_count_to_gconf (voice_cnt);
+    srs_write_drivers_to_gconf (drivers_);
+    srs_slist_free (drivers_, g_free);
+
+    g_ptr_array_add (driver_names, NULL);
+    g_ptr_array_add (voice_names, NULL);
+    src_voices->engine_drivers = (gchar**) g_ptr_array_free (driver_names, FALSE);
+    src_voices->engine_voices  = (gchar**) g_ptr_array_free (voice_names, FALSE);
+
+    srs_drivers_terminate (drivers);
+
+    return TRUE;
+}
