@@ -1133,7 +1133,11 @@ srl_report_event (SRLEvent *event)
 	ev = srl_last_events[i];
 	srl_last_events[i] = NULL;
 	if (ev)
+	{
+	    if (ev->type == SRL_EVENT_TEXT_CHANGED_INSERT && Accessible_getRole (ev->acc_ev->source) == SPI_ROLE_TERMINAL)
+		srl_notify_clients_obj (ev, SR_EVENT_SRO);
 	    srle_free (ev);
+	}
     }
 	    
     srl_last_events[priority] = srle_dup (event);
@@ -1812,7 +1816,8 @@ srl_get_context (Accessible *acc)
 		    if (label && container)
 		    {
 			if (Accessible_getRole (label) == SPI_ROLE_LABEL &&
-			    Accessible_getRole (container) == SPI_ROLE_FILLER)
+			    (Accessible_getRole (container) == SPI_ROLE_FILLER ||
+			     Accessible_getRole (container) == SPI_ROLE_PANEL))
 			{
 			    gchar *name = Accessible_getName (label);
 			    if (name && name[0])
