@@ -1,7 +1,7 @@
 /* screen-review.c
  *
- * Copyright 2001, 2002 Sun Microsystems Inc.,
- * Copyright 2001, 2002 Ximian, Inc.
+ * Copyright 2001, 2002 Sun Microsystems, Inc.,
+ * Copyright 2001, 2002 BAUM Retec, A.G.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -20,7 +20,7 @@
  */
 
 /*screen-review.c
- * VERSION : 0.40 (on 0.33)
+ * VERSION : 0.44 (on 0.33)
  * FIXED/DONE:
  *	- no debug messages
  *	- vertical spaces working
@@ -52,6 +52,15 @@
  *	- add SOURCE (the Accessible object) to each AccCell so the user would 
  *	  press the strip sensors and would obtain information about the object
  *	  displayed in the current line at that particular location
+ *	- got rid of memory leaks that where present due to calls like:
+ *		string = g_strconcat (string, string2, NULL);
+ *	- got rid of including diffrent delimiters for diffrent roles
+ *	 (this was redundant as this information is not passed to the screen-review
+ *	 client. The client will have the information about the role and that the cell
+ * 	 contains a delimitor (id) inside the SRWAccCEll structure.
+ *	- replaced the name of DELIMITATOR with DELIMITER (this is indeed english ;-) ) 
+ *	- introduced the z-order in the TextChunk, for future use (in case of 
+ *	SRW_SCOPE_APPLICATION or SRW_SCOPE_DESKTOP scope.
  *FIXME/2DO
  *	- add images, scrollbars, etc (objects that don't have a text displayed on screen)
  *	 and figure it out how would be represented on braille
@@ -60,7 +69,11 @@
  *	- get rid of some of globals. Refactoring required
  *	- break the functions (some of them are too big and very hard to read) 
  *	- Q: what should be presented when the application is iconofied and how? 
- *	- A: [NAME OF APP]?
+ *	- A: NAME OF WINDOW (iconofoed).
+ *	- lines should be of SRWLines type OR better SRWLineList == GList
+ *	- elements should be of type SRWTextChunkList = GList
+ * 	- there is no use of the align_flags parameter in function 
+ *		(srw_lines_create_from_elements)
  */
 
 #include "config.h"
@@ -69,7 +82,23 @@
 #include "SRMessages.h"
 
 #include "screen-review.h"
-
+/*
+ *SPI_LAYER_INVALID: The layer cannot be determined or is somehow undefined.
+ *SPI_LAYER_BACKGROUND: Component belongs to the destop background.
+ *SPI_LAYER_CANVAS: Component is a canvas backdrop or drawing area.
+ *SPI_LAYER_WIDGET: Component is a 'normal' widget.
+ *SPI_LAYER_MDI: Component is drawn in the MDI layer and may have valid
+ *                         Z-information relative to other MDI-layer components.
+ *SPI_LAYER_POPUP: Component is in the popup layer, above other widgets and
+ *                         MDI components.
+ *SPI_LAYER_OVERLAY: Component is in the overlay plane - this value is reserved
+ *                         for future use.
+ *SPI_LAYER_WINDOW: Component is in the window layer and have valid Z-information
+ *                   relative to other window-layer components.
+ *SPI_LAYER_LAST_DEFINED: Used to determine the last valid value in the enum,
+ *                         should not be encountered.	
+ * layer1 < layer2 ->layer2 is ontop of layer 1 
+*/
 
 typedef struct _SRWBoundaryRect 
 {
@@ -94,6 +123,7 @@ typedef struct _SRWTextChunk
 	SRWBoundaryRect	 start_char_bounds;
  	SRWBoundaryRect	 end_char_bounds;
 	int 		 id;
+	int		 z_order;
 	int		 layer;
 	gboolean	 is_text;
 	gboolean	 is_focused;
@@ -172,7 +202,10 @@ typedef GList SRWElements;
 			                  (i)->clip_bounds.width) ) )
 
 #define IS_CLIPPING_CONTAINER(a) ( ( (a) != SPI_ROLE_PAGE_TAB ) && 	\
-				   ( (a) != SPI_ROLE_MENU) )
+				   ( (a) != SPI_ROLE_MENU) 	&&	\
+				   ( (a) != -1) )
+#define MAX_PIXELS_PER_COLUMNS	 10000
+#define MIN_PIXELS_PER_COLUMNS	 2
 /*______________________________</MACROS>_____________________________________*/
 
 /*______________________________< GLOBALS>____________________________________*/
@@ -181,7 +214,7 @@ static SRWElements	*elements 		= NULL;
 static SRWLines		*lines			= NULL;
 static GArray 		*lines_index 		= NULL;
 static gint		id 			= 0,
-			pixels_per_column 	= G_MAXINT;
+			pixels_per_column 	= MAX_PIXELS_PER_COLUMNS;
 static glong		align_flags		= 0;
 static SRWBoundaryRect 	clipping_rectangle;
 /*______________________________</GLOBALS>____________________________________*/
@@ -293,7 +326,7 @@ srw_boundary_rect_new0 (void)
 	int 		i;
 		
 	bounds = g_new0 (SRWBoundaryRect *, SPI_LAYER_LAST_DEFINED);
-	for (i = 1; i < SPI_LAYER_LAST_DEFINED; ++i) 
+	for (i = 0; i < SPI_LAYER_LAST_DEFINED; ++i) 
 	{
 		bounds[i] = g_new0 (SRWBoundaryRect, 1);
 		bounds[i]->is_clipped = FALSE;
@@ -312,7 +345,7 @@ srw_boundary_rect_clone (SRWBoundaryRect *bounds[])
 		
 	bounds_clone = 
 		g_new0 (SRWBoundaryRect *, SPI_LAYER_LAST_DEFINED);
-	for (i = 1; i < SPI_LAYER_LAST_DEFINED; ++i) 
+	for (i = 0; i < SPI_LAYER_LAST_DEFINED; ++i) 
 	{
 		bounds_clone[i] = g_new0 (SRWBoundaryRect, 1);
 		*bounds_clone[i] = *bounds[i];
@@ -326,7 +359,7 @@ srw_boundary_rect_free (SRWBoundaryRect **bounds_clone)
 {
 	int 	i;
 	
-	for (i = 1; i < SPI_LAYER_LAST_DEFINED; ++i) 
+	for (i = 0; i < SPI_LAYER_LAST_DEFINED; ++i) 
 	{
 		g_free (bounds_clone[i]);
 		bounds_clone[i] = NULL;
@@ -451,7 +484,6 @@ srw_acc_cell_free (SRWAccCell *acc_cell)
 	    g_free (acc_cell->ch);
 	    acc_cell->ch = NULL;
 	}
-	/*FIXME : free the SRObject*/
 	if (acc_cell->source)
 	{
 	    sro_release_reference (acc_cell->source);
@@ -552,7 +584,7 @@ srw_acc_line_from_string (SRWAccLine 	*acc_line,
 		    else
 			if ( (offset + i == start_offset) ||
 			    (i == (len - 1) ) )
-				acc_cell->id = SRW_DELIMITATOR_CELL;/*delimitator	*/
+				acc_cell->id = SRW_DELIMITER_CELL;/*delimiter*/
 #ifdef SRW_TEXT_CHUNK_UTF8_DEBUG
 		        fprintf (stderr,"\nch =%string index=%d id=%d", 
 					acc_cell->ch, 
@@ -585,6 +617,7 @@ srw_text_chunk_new0 (char 	*string,
 		SRWBoundaryRect	*clip_bounds,
 		SRWBoundaryRect	*text_bounds,
 		int		id,
+		int		z_order,
 		int		layer)
 {
     SRWTextChunk *text_chunk;
@@ -603,6 +636,8 @@ srw_text_chunk_new0 (char 	*string,
     if (text_bounds)
 	text_chunk->text_bounds = *text_bounds;
 	
+
+    text_chunk->z_order		= z_order;
     text_chunk->layer 		= layer;
     text_chunk->id 		= id;
     text_chunk->dummy 		= FALSE;
@@ -660,7 +695,8 @@ srw_text_chunk_clone (SRWTextChunk	*text_chunk)
 static SRWTextChunk *
 srw_text_chunk_from_accessible (Accessible	*accessible, 
 			        SRWBoundaryRect	*bounds,
-			        long 		offset)
+			        long 		offset,
+				int 		z_order)
 {
 	SRWTextChunk 		*text_chunk 	= NULL;
 	AccessibleText		*text 		= NULL;
@@ -687,6 +723,7 @@ srw_text_chunk_from_accessible (Accessible	*accessible,
 				      bounds,
 				      NULL,
 				      0,
+				      z_order,
 				      layer);	
 
 	if (accessible)
@@ -806,9 +843,12 @@ srw_text_chunk_from_accessible (Accessible	*accessible,
 		    text_chunk->start_offset = start;
 		    text_chunk->end_offset = end ;
 
-		    if (text_chunk->text_bounds.width > 0 && string && strlen (string) )
+		    if (text_chunk->text_bounds.width > 0 && string && g_utf8_strlen (string, -1) )
+		    {
 			pixels_per_column = MIN (text_chunk->text_bounds.width / g_utf8_strlen (string, -1),
 						pixels_per_column);
+			pixels_per_column = MAX (pixels_per_column, MIN_PIXELS_PER_COLUMNS);
+		    }
 		}
 		AccessibleText_unref (text);
 		no_need = FALSE;
@@ -828,6 +868,7 @@ srw_text_chunk_from_accessible (Accessible	*accessible,
 		    role == SPI_ROLE_COLOR_CHOOSER	||	
 		    role == SPI_ROLE_PAGE_TAB 		||
 		    role == SPI_ROLE_STATUS_BAR         ||
+		    role == SPI_ROLE_CHECK_BOX          ||
 		    text_chunk->layer == SPI_LAYER_WINDOW||
 		    text_chunk->layer == SPI_LAYER_POPUP
 
@@ -849,6 +890,14 @@ srw_text_chunk_from_accessible (Accessible	*accessible,
 			text_chunk->text_bounds = text_chunk->clip_bounds;
 			text_chunk->start_offset = 0;
 			text_chunk->end_offset = strlen (string);
+
+			if (text_chunk->text_bounds.width > 0 && string && g_utf8_strlen (string,-1) )
+			{
+			    pixels_per_column = MIN (text_chunk->text_bounds.width / g_utf8_strlen (string, -1),
+						pixels_per_column);
+			    pixels_per_column = MAX (pixels_per_column, MIN_PIXELS_PER_COLUMNS);
+			}						
+
 #ifdef SRW_TEXT_CHUNK_DEBUG
 		fprintf (stderr, "\n\nsrw_text_chunk_from_accessible: NOT text |%s| %20s",
 				    string, 
@@ -1087,6 +1136,7 @@ srw_text_chunk_get_clipped_string (SRWTextChunk	*chunk)
 #endif
 		s = g_strdup (temp_string);
 		SPI_freeString (temp_string);
+		temp_string = NULL;
 		if (s[0] == ' ')
 		    word_start++;
 #ifdef SRW_CLIPPING_DEBUG
@@ -1132,7 +1182,9 @@ srw_text_chunk_get_clipped_string (SRWTextChunk	*chunk)
 		    end_inside) 
 		{
 		    /* word is contained in bounds */
-		    string = g_strconcat (string, s, NULL);
+		    temp_string = g_strconcat (string, s, NULL);
+		    g_free (string);
+		    string = temp_string;
 		} 
 		else 
 		{ 
@@ -1148,9 +1200,13 @@ srw_text_chunk_get_clipped_string (SRWTextChunk	*chunk)
 				    MIN (word_end, 
 					 chunk->end_offset));
 			if (s)
-			    string = g_strconcat (string, 
+			{		
+			    temp_string = g_strconcat (string, 
 					      s, 
 					      NULL);
+			    g_free (string);
+			    string = temp_string;
+			}
 /*		    }
 */		
 		}
@@ -1184,7 +1240,7 @@ srw_text_chunk_pad_string (SRWTextChunk	*chunk,
 		       glong		*start_offset,
 		       const char 	*pad_chars)
 {/*HSP flags*/
-	char 	*s ="";
+	char 	*s = NULL, *temp_string = NULL;
 	char 	*cp;
 	char 	startbuf[6], 
 		padbuf[6], 
@@ -1193,6 +1249,8 @@ srw_text_chunk_pad_string (SRWTextChunk	*chunk,
 	static	glong	 leading = 0;
 	gint 	howmany;
 
+	s = g_strdup("");
+	
 	howmany = g_unichar_to_utf8 (g_utf8_get_char (pad_chars), startbuf);
 	startbuf[howmany] = '\0';
 	g_assert (howmany < 7 && howmany > 0);
@@ -1238,7 +1296,9 @@ srw_text_chunk_pad_string (SRWTextChunk	*chunk,
 	    /*0000 */
 	    while (offset < end_padding )/*-1*/ 
 	    {
-		s = g_strconcat (s, padbuf, NULL); /* could be more efficient */
+		temp_string = g_strconcat (s, padbuf, NULL); /* could be more efficient */
+		g_free (s);
+		s = temp_string;
 		++offset;
 	    }
 	}
@@ -1248,7 +1308,9 @@ srw_text_chunk_pad_string (SRWTextChunk	*chunk,
 	    /* add left delimitator*/
 	    offset ++ ;
 	    *start_offset = offset;
-	    s = g_strconcat (s, startbuf, NULL);
+	    temp_string = g_strconcat (s, startbuf, NULL);
+	    g_free (s);
+	    s = temp_string;
 	}
 	/*add fill between left part of the obj and location where text begins*/
         if  (align_flags & SRW_ALIGNF_HSP_ADD_EMBEDDED) 
@@ -1261,14 +1323,18 @@ srw_text_chunk_pad_string (SRWTextChunk	*chunk,
 		end_padding = 0;
 	    while (offset < end_padding) 
 	    {
-		s = g_strconcat (s, padbuf, NULL); /* could be more efficient */
+		temp_string = g_strconcat (s, padbuf, NULL); /* could be more efficient */
+		g_free (s);
+		s = temp_string;
 		++offset;
 	    }
 	}
 	/*add text : OK*/
 	if (chunk && string && strlen (string))
 	{
-	    s = g_strconcat (s, string, NULL);
+	    temp_string = g_strconcat (s, string, NULL);
+	    g_free (s);
+	    s = temp_string;
 	    offset += g_utf8_strlen (string, -1);
 	}
 
@@ -1282,15 +1348,20 @@ srw_text_chunk_pad_string (SRWTextChunk	*chunk,
 		end_padding = 0;	
 	    while (offset < end_padding )/*-1*/ 
 	    {
-		s = g_strconcat (s, padbuf, NULL); /* could be more efficient */
+		temp_string = g_strconcat (s, padbuf, NULL); /* could be more efficient */
+		g_free (s);
+		s = temp_string;
 		++offset;
 	    }
 	}
 	/*]*/
 	/* add right delimitator*/
 	if (chunk && string && strlen (string) )
-	    s = g_strconcat (s, endbuf, NULL);
-	
+	{
+	    temp_string = g_strconcat (s, endbuf, NULL);
+	    g_free (s);
+	    s = temp_string;
+	}
 	return s;
 }
 
@@ -1305,45 +1376,8 @@ srw_text_chunk_to_string (SRWTextChunk 	*chunk,
 	if (chunk /*&& chunk->string*/) 
 	{
 		s = srw_text_chunk_get_clipped_string (chunk);
-
-	/*	if (chunk->dummy)
-			s = srw_text_chunk_pad_string (chunk, s, offset, &start_offset, "   ");
-		else
-	*/	switch (chunk->clip_bounds.role)
-		{
-		    case SPI_ROLE_PUSH_BUTTON:
-			s = srw_text_chunk_pad_string (chunk, s, offset, &start_offset,"[ ]");
-			break;
-		    case SPI_ROLE_TEXT:
-		    	s = srw_text_chunk_pad_string (chunk, s, offset, &start_offset, "\" \"");
-			break;
-		    case SPI_ROLE_CHECK_BOX:
-		    	s = srw_text_chunk_pad_string (chunk, s, offset, &start_offset, "x |");
-			break;
-		    case SPI_ROLE_RADIO_BUTTON:
-		    	s = srw_text_chunk_pad_string (chunk, s, offset, &start_offset, "o |");
-			break;
-		    default:
-			if (chunk->layer == SPI_LAYER_WINDOW) 
-			{
-			    s = srw_text_chunk_pad_string (chunk, s, offset, &start_offset, "| |");
-
-			}
-			else
-			{
-			    if (chunk->layer == SPI_LAYER_POPUP) 
-				s = srw_text_chunk_pad_string (chunk, s, offset, &start_offset, "< >");		    
-			    else
-				s = srw_text_chunk_pad_string (chunk, s, offset, &start_offset, "   ");
-			}
-			break;
-		}
-		
-	}
-	else
-	{
-	    s = srw_text_chunk_pad_string (NULL, s, offset, &start_offset, "! !");
 	}
+	s = srw_text_chunk_pad_string (chunk, s, offset, &start_offset,"| |");
 /*
 	fprintf (stderr,"\npad string %s",s);
 */
@@ -1362,12 +1396,20 @@ srw_text_chunk_compare_layer (gconstpointer  a,
     
     if (element_a && element_b)
     {
-	rv =  (element_a->layer % SPI_LAYER_WINDOW) -
-	      (element_b->layer % SPI_LAYER_WINDOW);
-    	if (rv)
+	rv =  (element_a->z_order) -
+	      (element_b->z_order);
+    
+	if (rv)
 	    return rv;
 	else
-	    return ( element_a->id - element_b->id);
+	{
+	    rv =  (element_a->layer) -
+	    	  (element_b->layer);
+    	    if (rv)
+		return rv;
+	    else
+		return ( element_a->id - element_b->id);
+	}
     }
     else 
 	return -1;
@@ -1389,10 +1431,11 @@ srw_text_chunk_list_debug (GList	*iter)
 	{
 		chunk = (SRWTextChunk *)iter->data;
 		if (chunk)
-		fprintf (stderr, "\n%d Chunk %40s id %4d layer %4d, clip %4ld-%4ld [%4ld], text %4ld-%4ld [%4ld] \ns%d e%d baseline %d",
+		fprintf (stderr, "\n%d Chunk %40s id %4d, z_order %4d, layer %4d, clip %4ld-%4ld [%4ld], text %4ld-%4ld [%4ld] \ns%d e%d baseline %d",
 			 chunk->is_text,
 			 chunk->string,
 			 chunk->id,
+			 chunk->z_order,
 			 chunk->layer,
 			 chunk->clip_bounds.y,
 			 chunk->clip_bounds.y + chunk->clip_bounds.height,
@@ -1442,12 +1485,13 @@ srw_text_chunk_list_split_insert (GList		*chunk_list,
 	chunk_list = g_list_insert_before (chunk_list, 
 					   iter, 
 					   iter_copy);
-	srw_text_chunk_tail_clip (iter_copy, 
-			      chunk);
+	srw_text_chunk_tail_clip (iter_copy,	/*bottom*/ 
+			          chunk		/*top   */);
 	chunk_list = g_list_insert_before (chunk_list, 
 					   iter, 
 					   chunk);
-	srw_text_chunk_head_clip (iter_chunk, chunk);
+	srw_text_chunk_head_clip (iter_chunk,	/*bottom*/ 
+				  chunk		/*top   */);
 
 	return chunk_list;
 }
@@ -1497,8 +1541,8 @@ srw_text_chunk_list_head_clip (GList	*text_chunk_list,
 				     (SRWTextChunk *)prev->data)
 	    ) 
 	{
-	    srw_text_chunk_tail_clip ( (SRWTextChunk *)prev->data,
-				    chunk);
+	    srw_text_chunk_tail_clip ( (SRWTextChunk *)prev->data,/*bottom*/
+					chunk			 /*top    */);
 	}
 	
 	return text_chunk_list;
@@ -1530,8 +1574,8 @@ srw_text_chunk_list_clip_and_insert (GList	*text_chunk_list,
 	    /* check for clip with prev */
 	    /* if we split the prev */
 	    if (prev &&
-		TEXT_CHUNK_BOUNDS_WITHIN (chunk, 
-					  (SRWTextChunk *) prev->data)
+		TEXT_CHUNK_BOUNDS_WITHIN (chunk, /*top*/
+					  (SRWTextChunk *) prev->data)/*bottom*/
 		) 
 	    {
 		text_chunk_list = srw_text_chunk_list_split_insert (text_chunk_list,
@@ -1547,8 +1591,8 @@ srw_text_chunk_list_clip_and_insert (GList	*text_chunk_list,
 		    {
 			text_chunk_list = srw_text_chunk_list_split_insert ( 
 						text_chunk_list,
-					    	next, 
-					        chunk);
+					    	next/*bottom*/, 
+					        chunk/*top*/);
 		    } 
 		    else 
 		    {
@@ -1639,10 +1683,12 @@ srw_text_chunk_list_to_string (GList *iter,
 				SRWAccLine *acc_line,
 				gint line_empty )
 {
-	char *s = "|";
-	char *string;
+	char *s = NULL;
+	char *string, *temp_string = NULL;
 	SRWTextChunk *chunk = NULL;
 
+	s = g_strdup ("|");
+	
 	acc_line->is_empty = line_empty;
 	while (iter) 
 	{
@@ -1650,7 +1696,11 @@ srw_text_chunk_list_to_string (GList *iter,
 		if (chunk) {
 			string = srw_text_chunk_to_string (chunk, g_utf8_strlen (s, -1) , acc_line);
 			if (string)
-				s = g_strconcat (s, string, NULL);
+			{
+			    temp_string = g_strconcat (s, string, NULL);
+			    g_free (s);
+			    s = temp_string;
+			}
 			
 		}
 		else
@@ -1662,9 +1712,15 @@ srw_text_chunk_list_to_string (GList *iter,
 
 	string = srw_text_chunk_to_string (NULL, g_utf8_strlen (s, -1), acc_line);
 	if (string)
-		s = g_strconcat (s, string, NULL);
+	{
+	    temp_string = g_strconcat (s, string, NULL);
+	    g_free (s);
+	    s = temp_string;
+	}
 
-	s = g_strconcat (s,"|",NULL);
+	temp_string = g_strconcat (s,"|",NULL);
+	g_free (s);
+	s = temp_string;
 	return s;
 }
 
@@ -1904,8 +1960,8 @@ OC*/
 	    else
 		line->layer = chunk_crt->layer;
 	    lines = g_list_append (lines, (SRWLine *) line);
-/*	
-	    fprintf (stderr,"\n -------NEW LINE--------------------------");
+	
+/*	    fprintf (stderr,"\n -------NEW LINE--------------------------");
 	    fprintf (stderr,"\n line->y1 %d, line->y2 %d line->baseline %d line->layer %d line->is_focused %d",
 			    line->y1,
 			    line->y2,
@@ -1933,8 +1989,8 @@ OC*/
 		
 		line = srw_line_add_text_chunk (prev_line, 
 						chunk_crt);
-/*
-	    fprintf (stderr,"\n ----------SAME LINE------------------");
+
+/*	    fprintf (stderr,"\n ----------SAME LINE------------------");
 	    fprintf (stderr,"\n line->y1 %d, line->y2 %d line->baseline %d line->layer %d",
 			    line->y1,
 			    line->y2,
@@ -2117,7 +2173,9 @@ srw_lines_free (GList	*lines)
     int 		i = 0;
     SRWLine 		*line;
     SRWTextChunk	*chunk;
-
+    GList		*lines_aux = lines;
+    
+    
 #ifdef SRW_CLEAN_UP_DEBUG
     fprintf (stderr, "\n\n<FREE LINES> %p",lines);
 #endif
@@ -2152,7 +2210,7 @@ srw_lines_free (GList	*lines)
 #ifdef SRW_CLEAN_UP_DEBUG    
     fprintf (stderr, "\n\n</FREE LINES>");
 #endif
-    g_free (lines);
+    g_list_free (lines_aux);
     lines = NULL; 
 }
 /*____________________________< /SRWLines methods>______________________________*/
@@ -2175,7 +2233,7 @@ srw_elements_compare_text_chunk_y (gconstpointer	a,
 	     element_a->text_bounds.height * SRW_BASELINE_RATIO  - 
 	    (element_b->text_bounds.y + 
 	    element_b->text_bounds.height * SRW_BASELINE_RATIO);
-    	if (rv)
+    	if (rv)/*-Treshold -> + TRESHOLD - equal???*/
 	    return rv;
 	else
 	    return ( element_a->clip_bounds.x - element_b->clip_bounds.x);
@@ -2189,7 +2247,8 @@ srw_elements_from_accessible (Accessible 	*accessible,
 			      Accessible	*focused_accessible,
 			      SRWBoundaryRect	*parentClipBounds[],
 			      gboolean		parent_selected,
-			      int		parent_layer)
+			      int		toplevel_z_order,
+			      int 		parent_layer)
 {
 	Accessible 		*child;
 	AccessibleRole		role;
@@ -2203,7 +2262,7 @@ srw_elements_from_accessible (Accessible 	*accessible,
 				child_n,
 				layer = -1;	
 	long 			offset = 0;
-		
+
 	Accessible_ref (accessible);
         clip_bounds = srw_boundary_rect_clone (parentClipBounds);
 
@@ -2239,19 +2298,39 @@ srw_elements_from_accessible (Accessible 	*accessible,
     				text_chunk = srw_text_chunk_from_accessible 
 					    (accessible, 
 					     &bounds, 
-					     offset);
+					     offset,
+					     toplevel_z_order);
 				if (text_chunk)
 				{
+					gboolean only_spaces = TRUE;
+					gint i, cnt;
+						
 					if (accessible == focused_accessible)
 					{
 					    text_chunk->is_focused = TRUE;
 					}
 					offset = text_chunk->end_offset + 1;
-					if (text_chunk->string)
+					
+					/* WORKAROUND for bug 144404:
+					   a line of only spaces should be ignored 
+					   (treated as an empty line) */
+					if (text_chunk->string != NULL)
+					{
+					    cnt = strlen (text_chunk->string);   
+					    for (i = 0; i < cnt; i++)
+						if ((guint16)*(text_chunk->string + i) != 0x20 &&
+					    	    (guint16)*(text_chunk->string + i) != 0x160)   
+						{
+						    only_spaces = FALSE;
+						    break;
+						}
+					}	
+					if (text_chunk->string && !only_spaces)
 					{
 						id++;
 						text_chunk->id 		= id;
-
+						/*special case for the window-title :
+						 There is no other way to determine it */
 						if (text_chunk->layer == SPI_LAYER_WINDOW)
 						{
 						    child = Accessible_getChildAtIndex (accessible, 0);
@@ -2298,6 +2377,8 @@ srw_elements_from_accessible (Accessible 	*accessible,
 		} 
 		Accessible_unref (component);
 	    }
+	    if (AccessibleStateSet_contains (states, SPI_STATE_ICONIFIED))
+		return;
 	/*
 	 * we always descend into children in case they are in a higher layer
 	 * this can of course be optimized for the topmost layer...
@@ -2318,6 +2399,7 @@ srw_elements_from_accessible (Accessible 	*accessible,
 						  focused_accessible,
 						  clip_bounds,
 						  parent_selected,
+						  toplevel_z_order,
 						  parent_layer); 
 		    Accessible_unref (child);
 		}
@@ -2437,6 +2519,7 @@ srw_elements_from_accessible (Accessible 	*accessible,
 						  focused_accessible,
 						  clip_bounds,
 						  parent_selected,
+						  toplevel_z_order,
 						  parent_layer); 
 		Accessible_unref (child);			    			    
 
@@ -2486,69 +2569,93 @@ srw_elements_free (GList *elements)
 
 /*_______________________________<AUX methods>______________________________________*/
 GList *
-srw_get_toplevels (void)
+srw_get_toplevels 	(Accessible 	*focused_acc, 
+			 glong 		scope_flags)
 {
-    Accessible		*desktop, 
-			*app;
-    AccessibleStateSet	*states;
-    GList 		*toplevels 	= NULL, 
-			*actives 	= NULL, 
-			*iter;
-    int 		n_apps, 
-			n_toplevels,
-			app_n, 
-			toplevel_n;
-
-    /* how do we decide which desktop ? */
-    desktop = SPI_getDesktop (0);
-  
-    /* for each app */
-    n_apps = Accessible_getChildCount (desktop);
-    for (app_n = 0; app_n < n_apps; ++app_n) 
+    Accessible	*desktop, *crt;
+    GList 	*toplevels = NULL;
+			
+    int 	n_toplevels,
+		toplevel_n;
+    
+    
+    srl_return_val_if_fail (focused_acc, FALSE);
+    srl_return_val_if_fail (!Accessible_isApplication (focused_acc), FALSE);
+    
+    if (scope_flags & SRW_SCOPE_DESKTOP)
     {
-	/* for each toplevel in app */
-	app =  Accessible_getChildAtIndex (desktop, app_n);
-	if (app)
+	int n_apps = 0,
+	    app_n;
+	    	    
+	/*desktop 0 is always the active one*/
+    	desktop = SPI_getDesktop (0);
+	n_apps = Accessible_getChildCount (desktop);
+	for (app_n = 0; app_n < n_apps; ++app_n)
 	{
-	    n_toplevels = Accessible_getChildCount (app);
-	    for (toplevel_n = 0; toplevel_n < n_toplevels; ++toplevel_n) 
+	    Accessible *app = NULL;
+	    app = Accessible_getChildAtIndex (desktop, app_n);
+	    if (app)
 	    {
-	        Accessible *toplevel = Accessible_getChildAtIndex (app, toplevel_n);
-	        if (Accessible_isComponent (toplevel))
-		    toplevels = g_list_prepend (toplevels, toplevel);
-		else 
+		n_toplevels = Accessible_getChildCount (app);
+		for (toplevel_n = 0; toplevel_n < n_toplevels; ++toplevel_n)
 		{
-		    Accessible_unref (toplevel);
-		    fprintf (stderr, "warning, app toplevel not a component.\n");
-		}
-	    }
-	    Accessible_unref (app);
-	}    
-    }
-    Accessible_unref (desktop); 
+		    Accessible *toplevel = NULL;
+		    toplevel = Accessible_getChildAtIndex (app, toplevel_n);
+		    if (toplevel && Accessible_isComponent (toplevel) )
+		    {
+			toplevels = g_list_prepend (toplevels, toplevel);
+		    }
+		    else
+		    {
+			Accessible_unref (toplevel);
+			fprintf (stderr, "warning, app toplevel not a component\n");
+		    }
+		} /* end for (toplevel_n ...) */
+		Accessible_unref (app);
+	    } /* end if (app) */
+	} /*end for (app_n ...) */
 
-    /* sort: at the moment we don't have a good way to sort except to put actives on top */
-    for (iter = g_list_first (toplevels); iter; iter = iter->next) 
+	return toplevels;
+    }
+	
+    crt = focused_acc;
+    Accessible_ref (crt);
+    for ( ; ; )
     {
-	Accessible *toplevel =
-		  (Accessible *) iter->data;
-	states = Accessible_getStateSet (toplevel);
-	if (AccessibleStateSet_contains (states,
-				   SPI_STATE_ACTIVE)) 
+	Accessible *parent;
+	parent = Accessible_getParent (crt);
+	if (parent && Accessible_isApplication (parent))
 	{
-	    actives = g_list_prepend (actives, toplevel);
+	    if (scope_flags & SRW_SCOPE_WINDOW)
+	    {
+		toplevels = g_list_prepend (toplevels, crt);
+	    }
+	    if (scope_flags & SRW_SCOPE_APPLICATION)
+	    {
+		n_toplevels = Accessible_getChildCount (parent);
+	    	for (toplevel_n = 0; toplevel_n < n_toplevels; ++toplevel_n) 
+	    	{
+	        	Accessible *toplevel = Accessible_getChildAtIndex (parent, toplevel_n);
+			toplevels = g_list_prepend (toplevels, toplevel);
+		}
+		Accessible_unref (crt);
+	    }
+	    Accessible_unref (parent);
+	    
+	    return toplevels;
 	}
-	AccessibleStateSet_unref (states);
+	if (!parent)
+	{
+	    srl_warning ("no object wich is application in parent line");
+	    Accessible_unref (crt);
+	    return NULL;
+	}
+	Accessible_unref (crt);
+	crt = parent;
     }
-
-    for (iter = g_list_first (actives); iter; iter = actives->next) 
-    {
-	toplevels = g_list_remove (toplevels, iter->data); /* place at end */
-	toplevels = g_list_append (toplevels, iter->data);
-    }
-    g_list_free (actives);
-
-    return toplevels;
+	
+    srl_assert_not_reached ();
+    return NULL;
 }
 /*_______________________________</AUX methods>______________________________________*/
 
@@ -2563,20 +2670,22 @@ srw_get_toplevels (void)
 int
 screen_review_init (SRRectangle *clip_rectangle,
 		    SRObject	*focused_object,
-		    glong 	alignment_flags)
+		    glong 	alignment_flags,
+		    glong 	scope_flags)
 {
 
-    Accessible 		*toplevel, *focused_accessible;
+    Accessible 		*toplevel, *focused_accessible, *desktop;
     AccessibleComponent	*component;
     AccessibleStateSet	*states;
 
     SRWBoundaryRect	**clip_bounds;
-    SRWBoundaryRect 	toplevel_bounds;
+    SRWBoundaryRect 	toplevel_bounds, desktop_bounds;
 
     GList 		*toplevels 	= NULL,
 			*iter		= NULL;
     int 		i,
-			n_lines = 0;
+			n_lines = 0,
+			z_order = G_MAXINT;
 #ifdef SRW_BENCHMARKING
     GTimer 		*timer = g_timer_new ();
 #endif
@@ -2585,18 +2694,45 @@ screen_review_init (SRRectangle *clip_rectangle,
   /*new && init*/
     clip_bounds	 = srw_boundary_rect_new0 ();
     id = 0;
-    pixels_per_column = G_MAXINT;
+    pixels_per_column = MAX_PIXELS_PER_COLUMNS;
     lines_index = NULL;
-    clipping_rectangle.x 	= clip_rectangle->x;
-    clipping_rectangle.y 	= clip_rectangle->y;
-    clipping_rectangle.width 	= clip_rectangle->width;
-    clipping_rectangle.height 	= clip_rectangle->height;
+    if (scope_flags & SRW_SCOPE_WINDOW)
+    {
+	clipping_rectangle.x 		= clip_rectangle->x;
+	clipping_rectangle.y 		= clip_rectangle->y;
+	clipping_rectangle.width 	= clip_rectangle->width;
+	clipping_rectangle.height 	= clip_rectangle->height;
+    }
+    else
+    {
+	clipping_rectangle.x 		= desktop_bounds.x;
+	clipping_rectangle.y 		= desktop_bounds.y;
+	clipping_rectangle.width 	= desktop_bounds.width;
+	clipping_rectangle.height 	= desktop_bounds.height;
+    }
     align_flags			= alignment_flags;
 
+    /* get desktop's size in order to clip the toplevels against it*/    
+    desktop = SPI_getDesktop (0);
+    if (desktop && Accessible_isComponent (desktop))
+    {
+    	component = Accessible_getComponent (desktop);
+	AccessibleComponent_getExtents (component,
+	    			        &desktop_bounds.x,
+					&desktop_bounds.y,
+					&desktop_bounds.width,
+					&desktop_bounds.height,
+					SPI_COORD_TYPE_SCREEN);
+	AccessibleComponent_unref (component);
+	Accessible_unref (desktop);
+    }
+    
   /* for each toplevel, ending with the active one(s),
    * clip against children.
    */
-    toplevels = srw_get_toplevels ();
+    focused_accessible = sro_get_acc_at_index (focused_object, 0);
+
+    toplevels = srw_get_toplevels (focused_accessible, scope_flags);
     if (toplevels && g_list_first (toplevels))
     for (iter = g_list_first (toplevels); iter; iter = iter->next)
     {
@@ -2605,10 +2741,11 @@ screen_review_init (SRRectangle *clip_rectangle,
 	{
   	    /* make sure toplevel is visible and not iconified or shaded */
 	    states = Accessible_getStateSet (toplevel);
-	    if ( (AccessibleStateSet_contains (states, SPI_STATE_VISIBLE) &&
-	    	  !AccessibleStateSet_contains (states, SPI_STATE_ICONIFIED) ))
+	    /* iconofied windows are visible, too */
+	    if (AccessibleStateSet_contains (states, SPI_STATE_VISIBLE) )
 	    { 
 		component = Accessible_getComponent (toplevel);
+		z_order = AccessibleComponent_getMDIZOrder (component);
 		AccessibleComponent_getExtents (component,
 		    			        &toplevel_bounds.x,
 						&toplevel_bounds.y,
@@ -2616,43 +2753,30 @@ screen_review_init (SRRectangle *clip_rectangle,
 						&toplevel_bounds.height,
 						SPI_COORD_TYPE_SCREEN);
 		toplevel_bounds.is_empty = FALSE;
-		if ( (toplevel_bounds.x == clip_rectangle->x) &&
-		     (toplevel_bounds.y == clip_rectangle->y) &&
-		     (toplevel_bounds.width == clip_rectangle->width)&&
-		     (toplevel_bounds.height == clip_rectangle->height))
-		{
-		    if (toplevel_bounds.x < 0)
-		    {
-			toplevel_bounds.width += toplevel_bounds.x;
-		    	toplevel_bounds.x = 0;
-		    }
-		    if (toplevel_bounds.y < 0)
-		    {
-			toplevel_bounds.height += toplevel_bounds.y;
-			toplevel_bounds.y = 0;
-		    }
-		    for (i = 1; i < SPI_LAYER_LAST_DEFINED; ++i)
-		    {
-		        *clip_bounds[i] = toplevel_bounds;
-		    }
-		    focused_accessible = sro_get_acc_at_index (focused_object, 0);
-		    srw_elements_from_accessible (toplevel,
-						  focused_accessible,
-						  clip_bounds,
-						  TRUE,
-						  -1);
-#ifdef SRW_TEXT_CHUNK_DEBUG
-		    srw_text_chunk_list_debug (elements);
-#endif
-		    lines =  srw_lines_create_from_elements (elements, align_flags);
+		toplevel_bounds.role = -1;/*invalid role*/
+    		srw_boundary_rect_clip (&toplevel_bounds, &desktop_bounds);
 
+		for (i = 0; i < SPI_LAYER_LAST_DEFINED; ++i)
+		{
+		        *clip_bounds[i] = toplevel_bounds;
 		}
+		srw_elements_from_accessible (toplevel,
+					  focused_accessible,
+					  clip_bounds,
+					  TRUE,
+					  z_order,
+					  -1);
 		AccessibleComponent_unref (component);
 	    }/*states*/
 	    AccessibleStateSet_unref (states);
 	}/*isComponent*/
 	Accessible_unref (toplevel);
     }
+#ifdef SRW_TEXT_CHUNK_DEBUG
+	    srw_text_chunk_list_debug (elements);
+#endif
+
+    lines =  srw_lines_create_from_elements (elements, align_flags);
     if (lines)
     {
 	n_lines = srw_lines_get_n_lines (&lines, align_flags);
@@ -2755,7 +2879,7 @@ screen_review_get_focused_line (void)
     if (lines)
     {
 	iter = g_list_first (lines);
-	line_number = 0;
+	line_number = 1;
 	while (iter)
 	{
 
@@ -2770,8 +2894,14 @@ screen_review_get_focused_line (void)
 	}
     }
     else
-    /*start from top if there is no focused object*/
+    /* start from top if there is no focused object*/
 	return 1;
+    
+    /* the focused object is not a really visible object so it won't be in any
+    line; there is not focused_line, so start from top  */
+    if (line_number > g_list_length (lines) )
+	return 1;
+    
     return line_number;
 }
 
